# ruff: noqa: E501, PLR0915
from __future__ import annotations

types: dict[str, str] = {
    "AWS::ACMPCA::Certificate": "8095f49d3b9d3b1a",
    "AWS::ACMPCA::CertificateAuthority": "97b6ba94b51843fc",
    "AWS::ACMPCA::CertificateAuthorityActivation": "a62fb980b49a2961",
    "AWS::ACMPCA::Permission": "f334231645d9b9b9",
    "AWS::ARCZonalShift::AutoshiftObserverNotificationStatus": "806adfa6e5aba057",
    "AWS::ARCZonalShift::ZonalAutoshiftConfiguration": "a2125021f0082633",
    "AWS::AccessAnalyzer::Analyzer": "82e24174e0bf1bd5",
    "AWS::AmazonMQ::Broker": "c7ad1fe2fbeb2d1b",
    "AWS::AmazonMQ::Configuration": "9a9a64561c801ccb",
    "AWS::AmazonMQ::ConfigurationAssociation": "2343e0ffe1114046",
    "AWS::ApiGateway::Account": "1876f63604439391",
    "AWS::ApiGateway::ApiKey": "48205d4ffd7feba3",
    "AWS::ApiGateway::Authorizer": "a51c1c1fc1195550",
    "AWS::ApiGateway::BasePathMapping": "14bcf4ff90345f78",
    "AWS::ApiGateway::BasePathMappingV2": "361e364ef63cf6fd",
    "AWS::ApiGateway::ClientCertificate": "3c72d768c43d41a3",
    "AWS::ApiGateway::Deployment": "2ffb8f3345f190c2",
    "AWS::ApiGateway::DocumentationPart": "1383fef137de858b",
    "AWS::ApiGateway::DocumentationVersion": "8f19f440f5e47b9f",
    "AWS::ApiGateway::DomainName": "1b67859ef84a0247",
    "AWS::ApiGateway::DomainNameAccessAssociation": "a990644bdd54299e",
    "AWS::ApiGateway::DomainNameV2": "a0369b9dd79fc7c8",
    "AWS::ApiGateway::GatewayResponse": "43d430f7d4055e59",
    "AWS::ApiGateway::Method": "8d3568941b9c1cfe",
    "AWS::ApiGateway::Model": "80f210a42639b333",
    "AWS::ApiGateway::RequestValidator": "afc1110de614be93",
    "AWS::ApiGateway::Resource": "0b20e16c9c975c0d",
    "AWS::ApiGateway::RestApi": "7c13875334b62e8c",
    "AWS::ApiGateway::Stage": "56ecb180a441ee32",
    "AWS::ApiGateway::UsagePlan": "c9548a0a281386e5",
    "AWS::ApiGateway::UsagePlanKey": "309b26469128fd0a",
    "AWS::ApiGateway::VpcLink": "3c3c844926fba248",
    "AWS::ApiGatewayV2::Api": "fdc84407d8d6260e",
    "AWS::ApiGatewayV2::ApiGatewayManagedOverrides": "6852b176617879b0",
    "AWS::ApiGatewayV2::ApiMapping": "907de0790553e448",
    "AWS::ApiGatewayV2::Authorizer": "69107e1e01673765",
    "AWS::ApiGatewayV2::Deployment": "81174b55f308dad3",
    "AWS::ApiGatewayV2::DomainName": "87535a11b8340a18",
    "AWS::ApiGatewayV2::Integration": "dde532823abe537b",
    "AWS::ApiGatewayV2::IntegrationResponse": "a319db9e828555fa",
    "AWS::ApiGatewayV2::Model": "fcc09a7d8ffbd09c",
    "AWS::ApiGatewayV2::Route": "112b036b7d923f13",
    "AWS::ApiGatewayV2::RouteResponse": "3606f855415b8421",
    "AWS::ApiGatewayV2::RoutingRule": "b9e023c3c57c3c04",
    "AWS::ApiGatewayV2::Stage": "2903f59facc01e1c",
    "AWS::ApiGatewayV2::VpcLink": "e5ea6845d4e71e0b",
    "AWS::AppConfig::Application": "b607b28b7e719446",
    "AWS::AppConfig::ConfigurationProfile": "cf14987e3f150eb8",
    "AWS::AppConfig::Deployment": "a999b7bdf5b07cf5",
    "AWS::AppConfig::DeploymentStrategy": "e6d16ca1a1cd4c61",
    "AWS::AppConfig::Environment": "0c307beb222c782d",
    "AWS::AppConfig::Extension": "d798510534a27baf",
    "AWS::AppConfig::ExtensionAssociation": "85102348a3d001d3",
    "AWS::AppConfig::HostedConfigurationVersion": "4d2f292068fae1c6",
    "AWS::AppMesh::GatewayRoute": "f31fd0c34cd762b9",
    "AWS::AppMesh::Mesh": "375120e4ddc54cb5",
    "AWS::AppMesh::Route": "10f96fe755a3b866",
    "AWS::AppMesh::VirtualGateway": "ea8db60d40f51bd0",
    "AWS::AppMesh::VirtualNode": "bda61df49b15dd6b",
    "AWS::AppMesh::VirtualRouter": "1a8079d7b7fd931b",
    "AWS::AppMesh::VirtualService": "b6b6bd277d3b2302",
    "AWS::AppSync::Api": "b46594583c264d17",
    "AWS::AppSync::ApiCache": "d7a7fb82edccdfe2",
    "AWS::AppSync::ApiKey": "3d104ea0be48564b",
    "AWS::AppSync::ChannelNamespace": "194048f8c8383a17",
    "AWS::AppSync::DataSource": "f42f0eade43578bb",
    "AWS::AppSync::FunctionConfiguration": "002d7fdacf417d21",
    "AWS::AppSync::GraphQLApi": "2d5221ec6cdad6fe",
    "AWS::AppSync::GraphQLSchema": "b3409083f48d1b5b",
    "AWS::AppSync::Resolver": "81052e564e6d42f5",
    "AWS::AppSync::SourceApiAssociation": "8176278e728ac6a7",
    "AWS::ApplicationAutoScaling::ScalableTarget": "de001f7e2b117e91",
    "AWS::ApplicationAutoScaling::ScalingPolicy": "a86b9b958db4147e",
    "AWS::ApplicationInsights::Application": "4fb681541bc69d81",
    "AWS::Athena::DataCatalog": "72b7e9ce721875ee",
    "AWS::Athena::NamedQuery": "8adbac6f36c9aa6b",
    "AWS::Athena::PreparedStatement": "f136b10278378b55",
    "AWS::Athena::WorkGroup": "98125aa523006e5f",
    "AWS::AutoScaling::AutoScalingGroup": "1176577a6acfd8f9",
    "AWS::AutoScaling::LaunchConfiguration": "d4a1a18430ff4776",
    "AWS::AutoScaling::LifecycleHook": "95c6d4e044efc81a",
    "AWS::AutoScaling::ScalingPolicy": "3a71f2160c19e614",
    "AWS::AutoScaling::ScheduledAction": "ab00e084f1d921e6",
    "AWS::AutoScaling::WarmPool": "93498a4faffea6ae",
    "AWS::AutoScalingPlans::ScalingPlan": "0a636faf3e94411d",
    "AWS::Backup::BackupPlan": "72bcc201f012e665",
    "AWS::Backup::BackupSelection": "26040166156c899a",
    "AWS::Backup::BackupVault": "695e80a4aaec9275",
    "AWS::Batch::ComputeEnvironment": "c04685284080736b",
    "AWS::Batch::ConsumableResource": "8e12226640545a9a",
    "AWS::Batch::JobDefinition": "8a7ecae4430d86b2",
    "AWS::Batch::JobQueue": "330cfa7e67eefadc",
    "AWS::Batch::SchedulingPolicy": "06201d728045e9e4",
    "AWS::Batch::ServiceEnvironment": "72c42b7d08edd4dc",
    "AWS::Budgets::Budget": "ebac0f729cbbf76a",
    "AWS::Budgets::BudgetsAction": "e06dd2791075c9d6",
    "AWS::CDK::Metadata": "6c85ec990b9ac854",
    "AWS::CE::AnomalyMonitor": "3238fa7e318e3af7",
    "AWS::CE::AnomalySubscription": "5c01c255f3f4980b",
    "AWS::CE::CostCategory": "0ca417b146507094",
    "AWS::Cassandra::Keyspace": "0bd2067860b304f2",
    "AWS::Cassandra::Table": "cb4f8b935e3eae9c",
    "AWS::Cassandra::Type": "8aec046f867c1dce",
    "AWS::CertificateManager::Account": "403ae9aaf56cf1a4",
    "AWS::CertificateManager::Certificate": "169f6d71dc51fcc8",
    "AWS::Cloud9::EnvironmentEC2": "0d96d5a1a805b082",
    "AWS::CloudFormation::CustomResource": "e1f552bc7275a696",
    "AWS::CloudFormation::HookDefaultVersion": "ed13bf5a5bcf4773",
    "AWS::CloudFormation::HookTypeConfig": "8dac7817ef160a80",
    "AWS::CloudFormation::HookVersion": "8ddaf92d4ae6b0da",
    "AWS::CloudFormation::Macro": "8517053591f9eceb",
    "AWS::CloudFormation::ModuleDefaultVersion": "0b395493be154f0d",
    "AWS::CloudFormation::ModuleVersion": "f98c4b774da7cc16",
    "AWS::CloudFormation::ResourceDefaultVersion": "f614137749ff39d3",
    "AWS::CloudFormation::ResourceVersion": "cb8c2d939b790590",
    "AWS::CloudFormation::Stack": "089169583d0e826d",
    "AWS::CloudFormation::StackSet": "bf26be80cf86dbfd",
    "AWS::CloudFormation::WaitCondition": "e62c75075457f02f",
    "AWS::CloudFormation::WaitConditionHandle": "edba290fd70976a3",
    "AWS::CloudFront::CachePolicy": "c08b3271cc175f11",
    "AWS::CloudFront::CloudFrontOriginAccessIdentity": "d2e8d397beac5797",
    "AWS::CloudFront::Distribution": "161a48dce89d32bd",
    "AWS::CloudFront::KeyGroup": "7a9f609d809f1f8e",
    "AWS::CloudFront::OriginRequestPolicy": "93db3cc24c0fa4c1",
    "AWS::CloudFront::PublicKey": "b5cac84e7e953094",
    "AWS::CloudFront::RealtimeLogConfig": "b264602b5a4f383b",
    "AWS::CloudFront::ResponseHeadersPolicy": "d5eaaa7cf7b104dc",
    "AWS::CloudFront::StreamingDistribution": "b9728ac9dbd5af28",
    "AWS::CloudTrail::Trail": "49fc1232e5cd1c2b",
    "AWS::CloudWatch::Alarm": "c5552fedb35a08eb",
    "AWS::CloudWatch::AnomalyDetector": "13b8b743191bf50f",
    "AWS::CloudWatch::CompositeAlarm": "5c9a218af0588cb5",
    "AWS::CloudWatch::Dashboard": "423086b5b02bd126",
    "AWS::CloudWatch::InsightRule": "7b287284e7e98391",
    "AWS::CloudWatch::MetricStream": "7428a359e19d26c0",
    "AWS::CodeBuild::Project": "cabfc6f21a839ae8",
    "AWS::CodeBuild::ReportGroup": "b34ce85ee1e4a8da",
    "AWS::CodeBuild::SourceCredential": "b84e190d9ebbbfe1",
    "AWS::CodeCommit::Repository": "70fff8b988e25027",
    "AWS::CodeDeploy::Application": "eea8100bd279b205",
    "AWS::CodeDeploy::DeploymentConfig": "394f39ae21382747",
    "AWS::CodeDeploy::DeploymentGroup": "36c8d71f30ed60e4",
    "AWS::CodePipeline::CustomActionType": "ab3f9c5bbc8251df",
    "AWS::CodePipeline::Pipeline": "580b88b2161ff941",
    "AWS::CodePipeline::Webhook": "a6ad2a4dc85ed032",
    "AWS::Cognito::IdentityPool": "3df22057c8e2ef54",
    "AWS::Cognito::IdentityPoolPrincipalTag": "3ae793569104c4f6",
    "AWS::Cognito::IdentityPoolRoleAttachment": "2a5296f4ca5a2ff7",
    "AWS::Config::AggregationAuthorization": "8957a0832482c5e3",
    "AWS::Config::ConfigRule": "161d23fa95d2540a",
    "AWS::Config::ConfigurationAggregator": "6192663bb376a59c",
    "AWS::Config::ConfigurationRecorder": "e35cee148e38b939",
    "AWS::Config::DeliveryChannel": "4f7af4ff91e52582",
    "AWS::Config::OrganizationConfigRule": "1f49966ab4677d68",
    "AWS::Config::RemediationConfiguration": "1891f5fc168dbf44",
    "AWS::Config::StoredQuery": "c02d3b47f92dac62",
    "AWS::DLM::LifecyclePolicy": "0287ea6dffbb40a2",
    "AWS::DMS::Certificate": "89efe57b4144e9ab",
    "AWS::DMS::DataProvider": "7c7365975be25275",
    "AWS::DMS::Endpoint": "945bb565b81d0034",
    "AWS::DMS::EventSubscription": "fdd5de7c61bc5946",
    "AWS::DMS::InstanceProfile": "b289107706553b44",
    "AWS::DMS::MigrationProject": "b4b035d4b0f0b861",
    "AWS::DMS::ReplicationConfig": "563e9b573376b7ec",
    "AWS::DMS::ReplicationInstance": "e491c749b1a4069d",
    "AWS::DMS::ReplicationSubnetGroup": "38c929d04f7ce1ee",
    "AWS::DMS::ReplicationTask": "0b9ef1a765b40e51",
    "AWS::DataBrew::Dataset": "d3b92e5e742770a5",
    "AWS::DataBrew::Job": "c8448544ff8acc4b",
    "AWS::DataBrew::Project": "f5afe30316bff351",
    "AWS::DataBrew::Recipe": "6fa6e2d8322ea926",
    "AWS::DataBrew::Ruleset": "932632c822ce6169",
    "AWS::DataBrew::Schedule": "1995bc021a9e7e23",
    "AWS::DataPipeline::Pipeline": "f54867769cc566cf",
    "AWS::DataSync::Agent": "733ace3e7eff9e68",
    "AWS::DataSync::LocationAzureBlob": "02a252d30ebe9091",
    "AWS::DataSync::LocationEFS": "dc695ceb626bd6aa",
    "AWS::DataSync::LocationFSxLustre": "94324095d22e84c5",
    "AWS::DataSync::LocationFSxWindows": "7123c04a1159211f",
    "AWS::DataSync::LocationHDFS": "9b0576d7bd1858ba",
    "AWS::DataSync::LocationNFS": "bf12cdbf56d7c3b2",
    "AWS::DataSync::LocationObjectStorage": "65e47a263206c90e",
    "AWS::DataSync::LocationS3": "34db2462e6e07c08",
    "AWS::DataSync::LocationSMB": "a045adab4853018c",
    "AWS::DataSync::Task": "6a02fb3e9861f0e9",
    "AWS::DirectoryService::MicrosoftAD": "b23e6a2f55331f07",
    "AWS::DirectoryService::SimpleAD": "ac553e5bf250acca",
    "AWS::DocDB::DBCluster": "be6f39b91c56eff9",
    "AWS::DocDB::DBClusterParameterGroup": "3e017448a15e0f15",
    "AWS::DocDB::DBInstance": "253562f270b587ac",
    "AWS::DocDB::DBSubnetGroup": "ccb97a9452b2284d",
    "AWS::DocDB::EventSubscription": "d22815028d03f301",
    "AWS::DynamoDB::GlobalTable": "1789ed6c17b2b960",
    "AWS::DynamoDB::Table": "8e1902cddc19c9ad",
    "AWS::EC2::CapacityReservation": "d8685a879c65fa62",
    "AWS::EC2::CapacityReservationFleet": "3cec76abc326e6e7",
    "AWS::EC2::CustomerGateway": "5c3cdba89933c1e2",
    "AWS::EC2::DHCPOptions": "7794e3a54eaacb7a",
    "AWS::EC2::EC2Fleet": "2667c807051620b3",
    "AWS::EC2::EIP": "cc42df82cdc81e9c",
    "AWS::EC2::EIPAssociation": "a267820d6250f189",
    "AWS::EC2::EgressOnlyInternetGateway": "922c59e19b698eac",
    "AWS::EC2::FlowLog": "ea553ae6f4a71ce3",
    "AWS::EC2::GatewayRouteTableAssociation": "50b450990964d2e7",
    "AWS::EC2::Host": "ba21ffd03e701456",
    "AWS::EC2::IPAM": "6fc9a0643e75121d",
    "AWS::EC2::IPAMAllocation": "c4cba824ca02ce4f",
    "AWS::EC2::IPAMPool": "99dd938378f53c74",
    "AWS::EC2::IPAMPoolCidr": "873e7cc6e6afef96",
    "AWS::EC2::IPAMResourceDiscovery": "694c0afb626c30d8",
    "AWS::EC2::IPAMResourceDiscoveryAssociation": "fde93486402ddbe5",
    "AWS::EC2::IPAMScope": "35088850e9d32a1c",
    "AWS::EC2::Instance": "51763c48d327df98",
    "AWS::EC2::InstanceConnectEndpoint": "aff65ea38d1b25dc",
    "AWS::EC2::InternetGateway": "4c7c90cb3105c9d8",
    "AWS::EC2::IpPoolRouteTableAssociation": "071b5a1f00936f72",
    "AWS::EC2::KeyPair": "349deb28f79434e4",
    "AWS::EC2::LaunchTemplate": "2da58e06d7524dc5",
    "AWS::EC2::NatGateway": "bb97935c916b47f0",
    "AWS::EC2::NetworkAcl": "41c7ec1cea436cd8",
    "AWS::EC2::NetworkAclEntry": "d4fc9ce8669023f0",
    "AWS::EC2::NetworkInterface": "1e436db5d46542c6",
    "AWS::EC2::NetworkInterfaceAttachment": "0d96c6d7269bdfb0",
    "AWS::EC2::NetworkInterfacePermission": "ff24405331f855c9",
    "AWS::EC2::PlacementGroup": "8dca9ce7eb8b50ac",
    "AWS::EC2::PrefixList": "c53292c8686d14de",
    "AWS::EC2::Route": "0ff811157612151a",
    "AWS::EC2::RouteTable": "67bdbfcad069c66a",
    "AWS::EC2::SecurityGroup": "5390282384be7866",
    "AWS::EC2::SecurityGroupEgress": "840df778de9b4be2",
    "AWS::EC2::SecurityGroupIngress": "367136b2f03f428c",
    "AWS::EC2::SecurityGroupVpcAssociation": "f48680388ee894fe",
    "AWS::EC2::SnapshotBlockPublicAccess": "f6434397a8c7e382",
    "AWS::EC2::SpotFleet": "d389b8af6ecb0e0b",
    "AWS::EC2::Subnet": "a60b8dc0c7f50396",
    "AWS::EC2::SubnetCidrBlock": "473785ce782ba18f",
    "AWS::EC2::SubnetNetworkAclAssociation": "6b4f4042734e67b7",
    "AWS::EC2::SubnetRouteTableAssociation": "6b18db45c4ef5f57",
    "AWS::EC2::TrafficMirrorFilter": "d97085902d4e54ae",
    "AWS::EC2::TrafficMirrorFilterRule": "df0dc3904abb859c",
    "AWS::EC2::TrafficMirrorSession": "ec54e16ff79eaac0",
    "AWS::EC2::TrafficMirrorTarget": "562f502a60abccd6",
    "AWS::EC2::TransitGateway": "bdd0278ecaccd4ae",
    "AWS::EC2::TransitGatewayAttachment": "40111e00bf0df74a",
    "AWS::EC2::TransitGatewayConnect": "05caf42b9c40a843",
    "AWS::EC2::TransitGatewayConnectPeer": "27459a628d254042",
    "AWS::EC2::TransitGatewayMeteringPolicy": "4daf1939996f77df",
    "AWS::EC2::TransitGatewayMeteringPolicyEntry": "153b7c0d6fcca798",
    "AWS::EC2::TransitGatewayMulticastDomain": "8a9fdc16690320b7",
    "AWS::EC2::TransitGatewayMulticastDomainAssociation": "f650430e0bbdcce9",
    "AWS::EC2::TransitGatewayMulticastGroupMember": "fe8c827e156dc451",
    "AWS::EC2::TransitGatewayMulticastGroupSource": "95b20156fbc77eb0",
    "AWS::EC2::TransitGatewayPeeringAttachment": "2a33a8efb207e5e5",
    "AWS::EC2::TransitGatewayRoute": "41a8efed021fd377",
    "AWS::EC2::TransitGatewayRouteTable": "46974e4ed5fe86b7",
    "AWS::EC2::TransitGatewayRouteTableAssociation": "a28bde73d4a5cde6",
    "AWS::EC2::TransitGatewayRouteTablePropagation": "e21e0c3273d49afe",
    "AWS::EC2::TransitGatewayVpcAttachment": "9f86e47a59b89462",
    "AWS::EC2::VPC": "f08626e9e82df64f",
    "AWS::EC2::VPCBlockPublicAccessExclusion": "27a1b09ed6dc90f2",
    "AWS::EC2::VPCBlockPublicAccessOptions": "1227e027d1c429f9",
    "AWS::EC2::VPCCidrBlock": "2ab876850f9f6c40",
    "AWS::EC2::VPCDHCPOptionsAssociation": "f803bdbad7d930d1",
    "AWS::EC2::VPCEndpoint": "1ece9f120d9da74a",
    "AWS::EC2::VPCEndpointConnectionNotification": "4d7f63de5d4c3a2e",
    "AWS::EC2::VPCEndpointService": "150eafe6174fc552",
    "AWS::EC2::VPCEndpointServicePermissions": "5841ea22a0cbb143",
    "AWS::EC2::VPCGatewayAttachment": "1dc77043848e8e94",
    "AWS::EC2::VPCPeeringConnection": "e01a466d269170ed",
    "AWS::EC2::VPNConnection": "e436dfe2cab19660",
    "AWS::EC2::VPNConnectionRoute": "d50bddcdd0369e1a",
    "AWS::EC2::VPNGateway": "1e2f029e99547e1a",
    "AWS::EC2::VPNGatewayRoutePropagation": "5e03d41fa9dc15b9",
    "AWS::EC2::Volume": "3385669f1c48cc6c",
    "AWS::EC2::VolumeAttachment": "1b60a799a0e3ac35",
    "AWS::ECR::PullTimeUpdateExclusion": "8f73d42f66f83eba",
    "AWS::ECR::RegistryPolicy": "80d3a81e49275d76",
    "AWS::ECR::RegistryScanningConfiguration": "68012043447b437a",
    "AWS::ECR::ReplicationConfiguration": "dc2a377f75525dd4",
    "AWS::ECR::Repository": "d569f41026af3be8",
    "AWS::ECR::RepositoryCreationTemplate": "260745c34d057c07",
    "AWS::ECS::CapacityProvider": "5804cc1956f638f2",
    "AWS::ECS::Cluster": "3990111074eac8be",
    "AWS::ECS::ClusterCapacityProviderAssociations": "4b512d1fe30ffdd3",
    "AWS::ECS::PrimaryTaskSet": "d8ea9e906cb85b80",
    "AWS::ECS::Service": "75313434fba04f66",
    "AWS::ECS::TaskDefinition": "ac3c5ff140cb5d7d",
    "AWS::ECS::TaskSet": "93284b06523a426d",
    "AWS::EFS::AccessPoint": "2f62161de474e688",
    "AWS::EFS::FileSystem": "1adec99cf2f47af3",
    "AWS::EFS::MountTarget": "eace751bcbd80a76",
    "AWS::EKS::AccessEntry": "6eb6ba186799e7f3",
    "AWS::EKS::Addon": "f27379fa02ce1e45",
    "AWS::EKS::Cluster": "f17916cc287d09df",
    "AWS::EKS::FargateProfile": "71e821ce6e609193",
    "AWS::EKS::IdentityProviderConfig": "77b807192827540b",
    "AWS::EKS::Nodegroup": "287f4d78035a9e03",
    "AWS::EKS::PodIdentityAssociation": "d4249aaeb9409683",
    "AWS::EMR::Cluster": "ac0466c8197ef416",
    "AWS::EMR::InstanceFleetConfig": "7c6308bb9ea61ce8",
    "AWS::EMR::InstanceGroupConfig": "57ea4fa9b7b6f658",
    "AWS::EMR::SecurityConfiguration": "59f2b8145f1762e6",
    "AWS::EMR::Step": "816de91a3e21c0a7",
    "AWS::EMR::Studio": "5562dc4dee066fd3",
    "AWS::EMR::StudioSessionMapping": "68073ac17f83b5d4",
    "AWS::EMRContainers::VirtualCluster": "d56c1799bc259539",
    "AWS::EMRServerless::Application": "b8f0bd50831c9f1e",
    "AWS::ElastiCache::CacheCluster": "1220b6bbd633ae68",
    "AWS::ElastiCache::GlobalReplicationGroup": "2af3062bf5b2cf7f",
    "AWS::ElastiCache::ParameterGroup": "b0e1dec3cd05f739",
    "AWS::ElastiCache::ReplicationGroup": "59defd83ba1cb2f2",
    "AWS::ElastiCache::SecurityGroup": "075dfa59da275723",
    "AWS::ElastiCache::SecurityGroupIngress": "96d1b4a5bd438d25",
    "AWS::ElastiCache::ServerlessCache": "67ad14ea2425e853",
    "AWS::ElastiCache::SubnetGroup": "b331f1d336ff42bf",
    "AWS::ElastiCache::User": "6ade31b4f3953b51",
    "AWS::ElastiCache::UserGroup": "6a6c2c035fd2b2df",
    "AWS::ElasticBeanstalk::Application": "b0333be3e99bd8ce",
    "AWS::ElasticBeanstalk::ApplicationVersion": "9ffbca44711c927f",
    "AWS::ElasticBeanstalk::ConfigurationTemplate": "e3276863e80191ff",
    "AWS::ElasticBeanstalk::Environment": "d3a3171169a7d77e",
    "AWS::ElasticLoadBalancing::LoadBalancer": "0c104e32be4cf97e",
    "AWS::ElasticLoadBalancingV2::Listener": "5676e09eada66ea4",
    "AWS::ElasticLoadBalancingV2::ListenerCertificate": "3edd898ae1082797",
    "AWS::ElasticLoadBalancingV2::ListenerRule": "813e329d3a1f8231",
    "AWS::ElasticLoadBalancingV2::LoadBalancer": "f4f22fe5256925cc",
    "AWS::ElasticLoadBalancingV2::TargetGroup": "f7dbf6903e0a1263",
    "AWS::ElasticLoadBalancingV2::TrustStore": "7300a052ca504e65",
    "AWS::ElasticLoadBalancingV2::TrustStoreRevocation": "593a78e5ca9c2be3",
    "AWS::Elasticsearch::Domain": "c6414ef13f2fedbe",
    "AWS::EventSchemas::Discoverer": "26c475458e43d0a3",
    "AWS::EventSchemas::Registry": "e90bca17695d13aa",
    "AWS::EventSchemas::RegistryPolicy": "f522642b76624a13",
    "AWS::EventSchemas::Schema": "1a9db6b605a5eee0",
    "AWS::Events::ApiDestination": "24a4a911aa69266e",
    "AWS::Events::Archive": "95c9a00401f17bbf",
    "AWS::Events::Connection": "fbb3c9dc28c6727b",
    "AWS::Events::EventBus": "87abb79a2ea2a571",
    "AWS::Events::EventBusPolicy": "7cb0a2b18483bb52",
    "AWS::Events::Rule": "4b8efe4a7f81f66f",
    "AWS::FMS::NotificationChannel": "622ce9589b28d75d",
    "AWS::FMS::Policy": "3d3868b1e974d27b",
    "AWS::FMS::ResourceSet": "6fb04db2dcf50bf8",
    "AWS::FSx::DataRepositoryAssociation": "2c944a4b33c2d639",
    "AWS::FSx::FileSystem": "9741d33840560c55",
    "AWS::FSx::Snapshot": "656a8874092132a7",
    "AWS::FSx::StorageVirtualMachine": "b62638ce430b0942",
    "AWS::FSx::Volume": "e2217632738abb7f",
    "AWS::GameLift::Alias": "5c014b121a693be2",
    "AWS::GameLift::Build": "b9b218f4180db0c3",
    "AWS::GameLift::Fleet": "7b1de719fe632688",
    "AWS::GameLift::GameServerGroup": "5cc95b605dfa9939",
    "AWS::GameLift::GameSessionQueue": "aba577170457cc78",
    "AWS::GameLift::MatchmakingConfiguration": "d94ba396d6b23182",
    "AWS::GameLift::MatchmakingRuleSet": "f6bc950e7c01648c",
    "AWS::GameLift::Script": "d2777b50e200b1a3",
    "AWS::Glue::Classifier": "e64237d854ac7fec",
    "AWS::Glue::Connection": "687ff6bd75a9175e",
    "AWS::Glue::Crawler": "5232917448cb4257",
    "AWS::Glue::CustomEntityType": "b46c7cf573d72e3d",
    "AWS::Glue::DataCatalogEncryptionSettings": "e4d7c9e644714bf4",
    "AWS::Glue::DataQualityRuleset": "faf8cbb22cb87066",
    "AWS::Glue::Database": "6a65ca13bf420f12",
    "AWS::Glue::DevEndpoint": "e620bcbf039ad200",
    "AWS::Glue::Job": "57e8fea46c9329ee",
    "AWS::Glue::MLTransform": "2c5a75b004c29815",
    "AWS::Glue::Partition": "b1116e6b758cc144",
    "AWS::Glue::Registry": "f29e746a5df78fb3",
    "AWS::Glue::Schema": "36f966bd9dff125a",
    "AWS::Glue::SchemaVersion": "70b6d2e7b702d49f",
    "AWS::Glue::SchemaVersionMetadata": "e531f6334f8eeda7",
    "AWS::Glue::SecurityConfiguration": "066142ba850e9899",
    "AWS::Glue::Table": "2528cdc058dacdf8",
    "AWS::Glue::Trigger": "efdf04c0265f5aff",
    "AWS::Glue::Workflow": "7889f86fa2b1958f",
    "AWS::Greengrass::ConnectorDefinition": "d81a2ea631976960",
    "AWS::Greengrass::ConnectorDefinitionVersion": "8f47953c935071e4",
    "AWS::Greengrass::CoreDefinition": "15c8baf34e5f8cad",
    "AWS::Greengrass::CoreDefinitionVersion": "2693a80c8060c381",
    "AWS::Greengrass::DeviceDefinition": "6ecc6ba96c8cc79d",
    "AWS::Greengrass::DeviceDefinitionVersion": "b502a71c845a7021",
    "AWS::Greengrass::FunctionDefinition": "8fd5402cad5a731f",
    "AWS::Greengrass::FunctionDefinitionVersion": "8a246bce38603a3b",
    "AWS::Greengrass::Group": "028bdeaf7ee3b4e1",
    "AWS::Greengrass::GroupVersion": "d150ea717ebf25a3",
    "AWS::Greengrass::LoggerDefinition": "ca08580793bf92db",
    "AWS::Greengrass::LoggerDefinitionVersion": "ea84d8c626f12182",
    "AWS::Greengrass::ResourceDefinition": "346ff6094dda5ee5",
    "AWS::Greengrass::ResourceDefinitionVersion": "3162bdad8992ee5c",
    "AWS::Greengrass::SubscriptionDefinition": "b24ced07fcb36f95",
    "AWS::Greengrass::SubscriptionDefinitionVersion": "1bc9c83f06823a39",
    "AWS::GreengrassV2::ComponentVersion": "66c2ae473cdf806c",
    "AWS::GreengrassV2::Deployment": "113077f9583c552f",
    "AWS::GuardDuty::Detector": "609c5ab9768f30a4",
    "AWS::GuardDuty::Filter": "3fd40922801a3b6a",
    "AWS::GuardDuty::IPSet": "82e2140b58b00302",
    "AWS::GuardDuty::Master": "5115377f51b894ca",
    "AWS::GuardDuty::Member": "4410bf4350c9fdb8",
    "AWS::GuardDuty::PublishingDestination": "95c6a8d647a77c52",
    "AWS::GuardDuty::ThreatIntelSet": "09915bee5770f4c9",
    "AWS::IAM::AccessKey": "7940f64975e793da",
    "AWS::IAM::Group": "8c9facb1f13f9457",
    "AWS::IAM::GroupPolicy": "58e37eeb095661a7",
    "AWS::IAM::InstanceProfile": "bf26619b978615f5",
    "AWS::IAM::ManagedPolicy": "d409538819ed546a",
    "AWS::IAM::OIDCProvider": "37a9a0d904b0c411",
    "AWS::IAM::Policy": "675bb1c6e349b968",
    "AWS::IAM::Role": "6b0320b7e7993d77",
    "AWS::IAM::RolePolicy": "3b144b67628ff5fc",
    "AWS::IAM::SAMLProvider": "a6f712f282976196",
    "AWS::IAM::ServerCertificate": "5840107f711b833c",
    "AWS::IAM::ServiceLinkedRole": "54931bc1a5b991c4",
    "AWS::IAM::User": "571fa9a137c01c78",
    "AWS::IAM::UserPolicy": "be8174f28c4bc37f",
    "AWS::IAM::UserToGroupAddition": "913f67f68af9fd37",
    "AWS::IAM::VirtualMFADevice": "7187a01cb3dd862f",
    "AWS::IdentityStore::Group": "1b0e7a9a2bec57b5",
    "AWS::IdentityStore::GroupMembership": "d010a01463729806",
    "AWS::ImageBuilder::Component": "c686558ba853a496",
    "AWS::ImageBuilder::ContainerRecipe": "882a5a874ec51f81",
    "AWS::ImageBuilder::DistributionConfiguration": "7b34e1b0b9958d96",
    "AWS::ImageBuilder::Image": "7eaad9cf0d30061e",
    "AWS::ImageBuilder::ImagePipeline": "8ea2eca13ceb1fc8",
    "AWS::ImageBuilder::ImageRecipe": "83c0121011689366",
    "AWS::ImageBuilder::InfrastructureConfiguration": "703bdd9fb961cbcd",
    "AWS::ImageBuilder::Workflow": "ec02974c9f6e2cf5",
    "AWS::IoT::AccountAuditConfiguration": "1df6ea7b44871f0a",
    "AWS::IoT::Authorizer": "c1bb226efdbe06df",
    "AWS::IoT::BillingGroup": "7356122caadbf1e6",
    "AWS::IoT::CACertificate": "777396c1bdf7b7e9",
    "AWS::IoT::Certificate": "9e87d1584bdfa627",
    "AWS::IoT::CertificateProvider": "ba58dd700876d1d2",
    "AWS::IoT::Command": "85dfdbc856bd6329",
    "AWS::IoT::CustomMetric": "95abcac33406bc29",
    "AWS::IoT::Dimension": "7abc67aee9bdb210",
    "AWS::IoT::DomainConfiguration": "bb375012e8bd7a84",
    "AWS::IoT::EncryptionConfiguration": "04291b59f537a48b",
    "AWS::IoT::FleetMetric": "fb5c21ce7d031532",
    "AWS::IoT::JobTemplate": "eae048ed2035c0e7",
    "AWS::IoT::Logging": "519a05631e6736c9",
    "AWS::IoT::MitigationAction": "35f9780b4ce4f2d1",
    "AWS::IoT::Policy": "0a051e10c89375bb",
    "AWS::IoT::PolicyPrincipalAttachment": "df0e6a915a42650b",
    "AWS::IoT::ProvisioningTemplate": "d9b34886a99d4afb",
    "AWS::IoT::ResourceSpecificLogging": "3472fe08f73d78f7",
    "AWS::IoT::RoleAlias": "47d4d78a34aa22b4",
    "AWS::IoT::ScheduledAudit": "985f3adc80a1fdc5",
    "AWS::IoT::SecurityProfile": "4f77596a1749304b",
    "AWS::IoT::SoftwarePackage": "6c7d8d10102d647b",
    "AWS::IoT::SoftwarePackageVersion": "09a582ea3f29b1d3",
    "AWS::IoT::Thing": "87d43589be8f9d6e",
    "AWS::IoT::ThingGroup": "7fa472878da23f08",
    "AWS::IoT::ThingPrincipalAttachment": "cb2db377e7001773",
    "AWS::IoT::ThingType": "472326c990487efa",
    "AWS::IoT::TopicRule": "b031e429916b85bb",
    "AWS::IoT::TopicRuleDestination": "1113af4de44b3cb0",
    "AWS::IoTAnalytics::Channel": "4b80a172cab89c05",
    "AWS::IoTAnalytics::Dataset": "d85b607a0e28e682",
    "AWS::IoTAnalytics::Datastore": "36a94ac3ef26724c",
    "AWS::IoTAnalytics::Pipeline": "06cacb1b1fc74fd8",
    "AWS::IoTEvents::AlarmModel": "9663bd41a6ba5243",
    "AWS::IoTEvents::DetectorModel": "0527d423b1402c41",
    "AWS::IoTEvents::Input": "30f33c5450a5de4f",
    "AWS::IoTSiteWise::AccessPolicy": "45b9535962dd9e69",
    "AWS::IoTSiteWise::Asset": "0780918734a86092",
    "AWS::IoTSiteWise::AssetModel": "9837c85953c5875f",
    "AWS::IoTSiteWise::Dashboard": "b93236063adb4877",
    "AWS::IoTSiteWise::Gateway": "61f283dfa0cd3bb8",
    "AWS::IoTSiteWise::Portal": "a342ed3aaa4d8dbd",
    "AWS::IoTSiteWise::Project": "3c3bebc705ae8134",
    "AWS::IoTTwinMaker::ComponentType": "4047916d6f25a52d",
    "AWS::IoTTwinMaker::Entity": "87cfb0f09e609346",
    "AWS::IoTTwinMaker::Scene": "fef394e5c8e7cd21",
    "AWS::IoTTwinMaker::SyncJob": "61149eafc0e0b523",
    "AWS::IoTTwinMaker::Workspace": "8ac573a855e809f6",
    "AWS::KMS::Alias": "28f10f922a4d194f",
    "AWS::KMS::Key": "c70e396ee12ddb65",
    "AWS::KMS::ReplicaKey": "903f43e74c8d8853",
    "AWS::KafkaConnect::Connector": "08b81c973eb812dd",
    "AWS::KafkaConnect::CustomPlugin": "f04fc57818e8ddb4",
    "AWS::KafkaConnect::WorkerConfiguration": "8a9b885926e8a00c",
    "AWS::Kinesis::ResourcePolicy": "39996915810e234a",
    "AWS::Kinesis::Stream": "e6126822cd48b2b0",
    "AWS::Kinesis::StreamConsumer": "7ce125c877a406e0",
    "AWS::KinesisAnalytics::Application": "50ae263f846c3862",
    "AWS::KinesisAnalytics::ApplicationOutput": "639fc1050aa8a059",
    "AWS::KinesisAnalytics::ApplicationReferenceDataSource": "8cda34a806d77b86",
    "AWS::KinesisAnalyticsV2::Application": "0c3ba846a178e940",
    "AWS::KinesisAnalyticsV2::ApplicationCloudWatchLoggingOption": "e07193f9ce1d04dc",
    "AWS::KinesisAnalyticsV2::ApplicationOutput": "14bce3d20b063bd9",
    "AWS::KinesisAnalyticsV2::ApplicationReferenceDataSource": "f9ca29a27845c82d",
    "AWS::KinesisFirehose::DeliveryStream": "6e5b365496a735c9",
    "AWS::LakeFormation::DataCellsFilter": "6ac217ac96918929",
    "AWS::LakeFormation::DataLakeSettings": "d9b68c11de60cdd0",
    "AWS::LakeFormation::Permissions": "062fc8932c9970c4",
    "AWS::LakeFormation::PrincipalPermissions": "89fd48f71918cde3",
    "AWS::LakeFormation::Resource": "64382446e2262067",
    "AWS::LakeFormation::Tag": "e6965562c3f7e5bb",
    "AWS::LakeFormation::TagAssociation": "22d7f7ee98ca2b17",
    "AWS::Lambda::Alias": "88c6e095730810db",
    "AWS::Lambda::EventInvokeConfig": "c30b1c987d66f67f",
    "AWS::Lambda::EventSourceMapping": "a3469723a672c99b",
    "AWS::Lambda::Function": "a0138839187c3dd7",
    "AWS::Lambda::LayerVersion": "f00fbc621c084236",
    "AWS::Lambda::LayerVersionPermission": "a46f7800bdd2c38c",
    "AWS::Lambda::Permission": "a43a5328bea4f6ad",
    "AWS::Lambda::Url": "a5fe082b690f22c9",
    "AWS::Lambda::Version": "e7fcd54deb138582",
    "AWS::LaunchWizard::Deployment": "2532c1afdd9c1f65",
    "AWS::Logs::AccountPolicy": "b8c495155e0407f4",
    "AWS::Logs::Delivery": "ddc55d6dc002371f",
    "AWS::Logs::DeliveryDestination": "30b61a3515971eda",
    "AWS::Logs::DeliverySource": "67b64320b397d6cd",
    "AWS::Logs::Destination": "ff1e6b020f2cf8e7",
    "AWS::Logs::LogGroup": "7144d581f2e97160",
    "AWS::Logs::LogStream": "bb97d1fb38ff99db",
    "AWS::Logs::MetricFilter": "9a1d875b35cd0f81",
    "AWS::Logs::QueryDefinition": "7b82b12db189c1d9",
    "AWS::Logs::ResourcePolicy": "d7898253a7730ac3",
    "AWS::Logs::SubscriptionFilter": "57635d7dc5bd3e61",
    "AWS::MSK::BatchScramSecret": "ff2b807a577495b0",
    "AWS::MSK::Cluster": "e0fb0b1582e5f892",
    "AWS::MSK::ClusterPolicy": "b60740da22ea8c04",
    "AWS::MSK::Configuration": "61dbb22c2b6cbbb1",
    "AWS::MSK::Replicator": "5b655e344006b494",
    "AWS::MSK::VpcConnection": "6aeb07916b90c80a",
    "AWS::MWAA::Environment": "c28ab3624f1d638c",
    "AWS::MemoryDB::ACL": "92db73d1650765a6",
    "AWS::MemoryDB::Cluster": "6b7ebaa919714fa9",
    "AWS::MemoryDB::ParameterGroup": "694beb8bcf66b646",
    "AWS::MemoryDB::SubnetGroup": "058d84a44c7cdeb2",
    "AWS::MemoryDB::User": "2c12ec2e734bb40c",
    "AWS::Neptune::DBCluster": "8d9215c18d4689e5",
    "AWS::Neptune::DBClusterParameterGroup": "d3f072d3d16a7a5f",
    "AWS::Neptune::DBInstance": "40928fad97bbb2cd",
    "AWS::Neptune::DBParameterGroup": "6b2f9cd1b2702ccf",
    "AWS::Neptune::DBSubnetGroup": "c59d053f84ba853c",
    "AWS::Neptune::EventSubscription": "291cb01f3cbffb3e",
    "AWS::NetworkFirewall::Firewall": "e72dd64edb0563ec",
    "AWS::NetworkFirewall::FirewallPolicy": "23d185da5cfdceca",
    "AWS::NetworkFirewall::LoggingConfiguration": "24065655a3066a83",
    "AWS::NetworkFirewall::RuleGroup": "0a775a8261b19aad",
    "AWS::NetworkFirewall::TLSInspectionConfiguration": "c67eab10509c4b9d",
    "AWS::NetworkFirewall::VpcEndpointAssociation": "5862c6e91def980a",
    "AWS::Oam::Link": "4d4e114e0f615d82",
    "AWS::Oam::Sink": "81957ea027e4c201",
    "AWS::OpenSearchService::Domain": "1c09878d526db02c",
    "AWS::OpsWorks::App": "508d500b32261320",
    "AWS::OpsWorks::ElasticLoadBalancerAttachment": "4902f6030931194b",
    "AWS::OpsWorks::Instance": "2d5afc2d419e057d",
    "AWS::OpsWorks::Layer": "c0f477e5b002651a",
    "AWS::OpsWorks::Stack": "29ffdc808f9c83e1",
    "AWS::OpsWorks::UserProfile": "c81a3392e657795c",
    "AWS::OpsWorks::Volume": "2de519e2f08e58bd",
    "AWS::Organizations::Account": "5e4fa45032c79057",
    "AWS::Organizations::Organization": "ef5f3cbaa91637bd",
    "AWS::Organizations::OrganizationalUnit": "97d519f28c43e840",
    "AWS::Organizations::Policy": "11262354a02bb72d",
    "AWS::Organizations::ResourcePolicy": "44e0f38bebacad2e",
    "AWS::Personalize::Dataset": "9e7cafcfcba638cb",
    "AWS::Personalize::DatasetGroup": "d4a5c5722f39080e",
    "AWS::Personalize::Schema": "52e43b4875fdf0f3",
    "AWS::Personalize::Solution": "bf8a32c997c789a9",
    "AWS::Pipes::Pipe": "052457cd8332df71",
    "AWS::QuickSight::Analysis": "9ed141915b114fbd",
    "AWS::QuickSight::CustomPermissions": "ac2895cdbe3ee0f3",
    "AWS::QuickSight::Dashboard": "c75bfb3b0c668dfa",
    "AWS::QuickSight::DataSet": "0e9e94a2fd9e8f95",
    "AWS::QuickSight::DataSource": "2c06940063cc0519",
    "AWS::QuickSight::Folder": "4a9e4af92dbcc718",
    "AWS::QuickSight::RefreshSchedule": "0e848e5d85a17427",
    "AWS::QuickSight::Template": "9f99a52291094b16",
    "AWS::QuickSight::Theme": "c85a23cc7ca1d55e",
    "AWS::QuickSight::VPCConnection": "96804832324c84da",
    "AWS::RAM::Permission": "740a6e8ada63a5a3",
    "AWS::RAM::ResourceShare": "b4b5ff31bc5f95b0",
    "AWS::RDS::DBCluster": "9c7540d3d4699497",
    "AWS::RDS::DBClusterParameterGroup": "4e3460620664c07f",
    "AWS::RDS::DBInstance": "b7cc196bc1fe0b0d",
    "AWS::RDS::DBParameterGroup": "d29cf38b861f6048",
    "AWS::RDS::DBProxy": "6a3bd3de0c3a9c96",
    "AWS::RDS::DBProxyEndpoint": "db0fcba23f51c12d",
    "AWS::RDS::DBProxyTargetGroup": "8ee69ccb4de53286",
    "AWS::RDS::DBSecurityGroup": "42f555e782cd9946",
    "AWS::RDS::DBSecurityGroupIngress": "1a14ad6e73c0b6b7",
    "AWS::RDS::DBShardGroup": "df6b36c7e099595a",
    "AWS::RDS::DBSubnetGroup": "4a2ab41cbcc2adb8",
    "AWS::RDS::EventSubscription": "85bf3af749d312f0",
    "AWS::RDS::GlobalCluster": "a87d030f06c19d47",
    "AWS::RDS::Integration": "f7415a6a9c4a11da",
    "AWS::RDS::OptionGroup": "d89f528bd7bdd696",
    "AWS::Rbin::Rule": "6c9b3a0b86f4c148",
    "AWS::Redshift::Cluster": "bd4c5acbf9c19128",
    "AWS::Redshift::ClusterParameterGroup": "37da89c6de1fa7a4",
    "AWS::Redshift::ClusterSecurityGroup": "6a629309a94b6faa",
    "AWS::Redshift::ClusterSecurityGroupIngress": "0079cffa4eeb3685",
    "AWS::Redshift::ClusterSubnetGroup": "15d4337dca25524a",
    "AWS::Redshift::EndpointAccess": "b9824e20c8b887c4",
    "AWS::Redshift::EndpointAuthorization": "e72be424d109da1d",
    "AWS::Redshift::EventSubscription": "53db04ab09eaba8e",
    "AWS::Redshift::Integration": "a386ce60053e2aa9",
    "AWS::Redshift::ScheduledAction": "51bcaaee4f45725e",
    "AWS::RedshiftServerless::Namespace": "5713f5387741d15b",
    "AWS::RedshiftServerless::Snapshot": "826c5d08e248c956",
    "AWS::RedshiftServerless::Workgroup": "d725f73083591093",
    "AWS::ResourceGroups::Group": "c777809feaeb0e2b",
    "AWS::RolesAnywhere::CRL": "e5f5be456a9e1f7c",
    "AWS::RolesAnywhere::Profile": "efa966abbef1d995",
    "AWS::RolesAnywhere::TrustAnchor": "6b14ddb523aec5e8",
    "AWS::Route53::CidrCollection": "2332f11a72446133",
    "AWS::Route53::HealthCheck": "f075f8dd247254c7",
    "AWS::Route53::HostedZone": "29b2fce456164442",
    "AWS::Route53::RecordSet": "c01e9eccec2e3b83",
    "AWS::Route53::RecordSetGroup": "8300e7c48d942273",
    "AWS::Route53Profiles::Profile": "cb0a28f9bebd044e",
    "AWS::Route53Profiles::ProfileAssociation": "fa568e44f7d9e9f2",
    "AWS::Route53Profiles::ProfileResourceAssociation": "73d6b9da50afa75b",
    "AWS::Route53Resolver::FirewallDomainList": "76957613d7f85b1b",
    "AWS::Route53Resolver::FirewallRuleGroup": "c0eed6795a190c56",
    "AWS::Route53Resolver::FirewallRuleGroupAssociation": "d84c68a6ebbb5876",
    "AWS::Route53Resolver::ResolverConfig": "174be26547caaa42",
    "AWS::Route53Resolver::ResolverDNSSECConfig": "813fd8fe1e24c6e3",
    "AWS::Route53Resolver::ResolverEndpoint": "d0ad42bf85f59279",
    "AWS::Route53Resolver::ResolverQueryLoggingConfig": "78d814f813fdbcbf",
    "AWS::Route53Resolver::ResolverQueryLoggingConfigAssociation": "41e93f940f8bce93",
    "AWS::Route53Resolver::ResolverRule": "a5bf3dc0f2b44d80",
    "AWS::Route53Resolver::ResolverRuleAssociation": "3e5cdfc8643c083b",
    "AWS::S3::AccessGrant": "b1e7af0495a078b0",
    "AWS::S3::AccessGrantsInstance": "05d3e874b43bbad8",
    "AWS::S3::AccessGrantsLocation": "3a7be1727f630029",
    "AWS::S3::AccessPoint": "e8b95810dd8d6917",
    "AWS::S3::Bucket": "1aa877c9b86f9e89",
    "AWS::S3::BucketPolicy": "38ad5b8bd8062535",
    "AWS::S3::StorageLens": "e20ccca5b946adc3",
    "AWS::S3::StorageLensGroup": "a48c03e67d334c14",
    "AWS::S3Express::AccessPoint": "319fc2700104fd70",
    "AWS::S3Express::BucketPolicy": "79c6732ff6204662",
    "AWS::S3Express::DirectoryBucket": "06fdc76e1b66a21f",
    "AWS::S3ObjectLambda::AccessPoint": "2d6b2b82a9508114",
    "AWS::S3ObjectLambda::AccessPointPolicy": "fd6328c6c782223a",
    "AWS::SDB::Domain": "48c38397df169b43",
    "AWS::SNS::Subscription": "9426c559a25e9da6",
    "AWS::SNS::Topic": "dfcf55503343941a",
    "AWS::SNS::TopicInlinePolicy": "e47e6b028a7c3cc7",
    "AWS::SNS::TopicPolicy": "04470a92c9a1130c",
    "AWS::SQS::Queue": "4b3c7b55a266949b",
    "AWS::SQS::QueueInlinePolicy": "e4132d6c68dcbc6c",
    "AWS::SQS::QueuePolicy": "8d0c3b8154e71ff3",
    "AWS::SSM::Association": "ad263c9023f92e0c",
    "AWS::SSM::Document": "8b3a8b629d6edee1",
    "AWS::SSM::MaintenanceWindow": "54b31dcf040d1e3b",
    "AWS::SSM::MaintenanceWindowTarget": "5a6af4468fbb6f80",
    "AWS::SSM::MaintenanceWindowTask": "bdd479736cab2d77",
    "AWS::SSM::Parameter": "c247b226112c5ae5",
    "AWS::SSM::PatchBaseline": "43cce065b6753223",
    "AWS::SSM::ResourceDataSync": "f92ce1aa6674b7f2",
    "AWS::SSO::Application": "996ad386536ff750",
    "AWS::SSO::ApplicationAssignment": "ae6daf9e56a38982",
    "AWS::SSO::Assignment": "3e0b05f0a3cba8d8",
    "AWS::SSO::Instance": "12637265e4117853",
    "AWS::SSO::InstanceAccessControlAttributeConfiguration": "ccfedd9599046cb1",
    "AWS::SSO::PermissionSet": "e4212a96dd4b627d",
    "AWS::SageMaker::App": "a2a6c6e24ea97c09",
    "AWS::SageMaker::AppImageConfig": "c4676a45bc3fb01f",
    "AWS::SageMaker::CodeRepository": "d59f5cf4521ff1c2",
    "AWS::SageMaker::Domain": "d0c9a5567721ed89",
    "AWS::SageMaker::Endpoint": "73ec8bbe3002c627",
    "AWS::SageMaker::EndpointConfig": "7c3a65bd96c2e9a3",
    "AWS::SageMaker::FeatureGroup": "dac504afb40ef88a",
    "AWS::SageMaker::Image": "407ba0b85aaa0559",
    "AWS::SageMaker::ImageVersion": "307ec283c9c38549",
    "AWS::SageMaker::Model": "9757bf2bdc6a25ef",
    "AWS::SageMaker::ModelCard": "3eb6d59e80cf06f6",
    "AWS::SageMaker::ModelPackage": "14150fce0eeff149",
    "AWS::SageMaker::MonitoringSchedule": "aa0115534ffe6a87",
    "AWS::SageMaker::NotebookInstance": "f222a6268560b3b6",
    "AWS::SageMaker::NotebookInstanceLifecycleConfig": "f0d21af114094fa4",
    "AWS::SageMaker::Pipeline": "34dda455906546ac",
    "AWS::SageMaker::ProcessingJob": "9bf6e74e7f3d53e1",
    "AWS::SageMaker::Project": "c10c82bc5a09a5dc",
    "AWS::SageMaker::StudioLifecycleConfig": "5b453bb7816eb95b",
    "AWS::SageMaker::UserProfile": "d2a4b7455b2cbd74",
    "AWS::SageMaker::Workteam": "d617e40122a8fe93",
    "AWS::Scheduler::Schedule": "e8b60514af25a837",
    "AWS::Scheduler::ScheduleGroup": "ef710d09dbb243bf",
    "AWS::SecretsManager::ResourcePolicy": "cb5d330bf15d9d5a",
    "AWS::SecretsManager::RotationSchedule": "1f88d2e8812cdf5d",
    "AWS::SecretsManager::Secret": "4817d08ab5bef439",
    "AWS::SecretsManager::SecretTargetAttachment": "3bf4404734c82308",
    "AWS::SecurityHub::AutomationRule": "f6c9ad12353e867b",
    "AWS::SecurityHub::ConfigurationPolicy": "acb80ff79b887cb5",
    "AWS::SecurityHub::DelegatedAdmin": "a328fc62a9f2f7e2",
    "AWS::SecurityHub::FindingAggregator": "7a4c95623ee48fcd",
    "AWS::SecurityHub::Hub": "9f6cb2d8967bfcf6",
    "AWS::SecurityHub::Insight": "1ec7254a3cf4d038",
    "AWS::SecurityHub::OrganizationConfiguration": "494d5bdaaa99a7a9",
    "AWS::SecurityHub::PolicyAssociation": "fe287d4170e40018",
    "AWS::SecurityHub::ProductSubscription": "a6a7b09a786ba544",
    "AWS::SecurityHub::SecurityControl": "062ab72de4bbcd1d",
    "AWS::SecurityHub::Standard": "a9a57a7527f5aff5",
    "AWS::ServiceCatalog::AcceptedPortfolioShare": "77373a56c0c4bd28",
    "AWS::ServiceCatalog::CloudFormationProduct": "662186534ff8e643",
    "AWS::ServiceCatalog::CloudFormationProvisionedProduct": "448a377417beb2e5",
    "AWS::ServiceCatalog::LaunchNotificationConstraint": "7df0b63dbae97812",
    "AWS::ServiceCatalog::LaunchRoleConstraint": "dfc41af30527c286",
    "AWS::ServiceCatalog::LaunchTemplateConstraint": "dcbb2e3cc9935c1d",
    "AWS::ServiceCatalog::Portfolio": "52a609bb3d2294b5",
    "AWS::ServiceCatalog::PortfolioPrincipalAssociation": "c1655e6cd9e94bc0",
    "AWS::ServiceCatalog::PortfolioProductAssociation": "39311ae1cb0b583e",
    "AWS::ServiceCatalog::PortfolioShare": "cc64bce13ce0fae6",
    "AWS::ServiceCatalog::ResourceUpdateConstraint": "7181855aed7169b4",
    "AWS::ServiceCatalog::StackSetConstraint": "34cef71a3b8b5ea3",
    "AWS::ServiceCatalog::TagOption": "0a7e7604f593ea6c",
    "AWS::ServiceCatalog::TagOptionAssociation": "932ff2b84208a648",
    "AWS::ServiceDiscovery::HttpNamespace": "986cb51cac464fe8",
    "AWS::ServiceDiscovery::Instance": "b4227e1dfa14a394",
    "AWS::ServiceDiscovery::PrivateDnsNamespace": "d78a2213969ef282",
    "AWS::ServiceDiscovery::PublicDnsNamespace": "da152fa1f7dce966",
    "AWS::ServiceDiscovery::Service": "aa107587d49c4b29",
    "AWS::StepFunctions::Activity": "b3156a89a45815e1",
    "AWS::StepFunctions::StateMachine": "f6e6ebffca5675b3",
    "AWS::StepFunctions::StateMachineAlias": "55cc66ee1f33c9f5",
    "AWS::StepFunctions::StateMachineVersion": "4467293a52255b78",
    "AWS::Synthetics::Canary": "895da8ad2bfe9dad",
    "AWS::Transfer::Agreement": "ed933cf6f3e7c136",
    "AWS::Transfer::Certificate": "f7478973edec7bd3",
    "AWS::Transfer::Connector": "5ff069df52b96462",
    "AWS::Transfer::Profile": "88c7478695f7c070",
    "AWS::Transfer::Server": "90b633961fce2a5f",
    "AWS::Transfer::User": "7dbec614b03c1346",
    "AWS::Transfer::WebApp": "09ed0f7d971d7cb8",
    "AWS::Transfer::Workflow": "657766d62aa7fb4e",
    "AWS::VerifiedPermissions::Policy": "f7e75c34791cf6a5",
    "AWS::VerifiedPermissions::PolicyStore": "5883732ea6547044",
    "AWS::VerifiedPermissions::PolicyTemplate": "067128efc0e8f2eb",
    "AWS::WAF::ByteMatchSet": "bdac58da17fc8f67",
    "AWS::WAF::IPSet": "e9b5a4840b312167",
    "AWS::WAF::Rule": "2eb380e895f6c1fe",
    "AWS::WAF::SizeConstraintSet": "7b4d9b8017b0288e",
    "AWS::WAF::SqlInjectionMatchSet": "392352f8be8c6258",
    "AWS::WAF::WebACL": "347318598f8787e2",
    "AWS::WAF::XssMatchSet": "ac4c34fbea311314",
    "AWS::WAFRegional::ByteMatchSet": "994da0cf8bd3e164",
    "AWS::WAFRegional::GeoMatchSet": "02fe4746f224ce16",
    "AWS::WAFRegional::IPSet": "44ebba581bb9b196",
    "AWS::WAFRegional::RateBasedRule": "4e4d503472dd02e8",
    "AWS::WAFRegional::RegexPatternSet": "bec049612963886e",
    "AWS::WAFRegional::Rule": "c52059662e2cd2fc",
    "AWS::WAFRegional::SizeConstraintSet": "2020060b93f50b79",
    "AWS::WAFRegional::SqlInjectionMatchSet": "e934a6dec5e807ed",
    "AWS::WAFRegional::WebACL": "375f1db4fc74a74e",
    "AWS::WAFRegional::WebACLAssociation": "f6cb657edbe28837",
    "AWS::WAFRegional::XssMatchSet": "d06f5257461f5e67",
    "AWS::WAFv2::IPSet": "d140014308da4470",
    "AWS::WAFv2::LoggingConfiguration": "219a7415589ec0bf",
    "AWS::WAFv2::RegexPatternSet": "71e5110f182eba06",
    "AWS::WAFv2::RuleGroup": "b1d084941142148b",
    "AWS::WAFv2::WebACL": "892990086386d387",
    "AWS::WAFv2::WebACLAssociation": "d0181fa0b0d2faac",
    "AWS::WorkSpaces::Workspace": "97435699d74bcbe8",
    "AWS::XRay::Group": "81cf1b3bfd81fc73",
    "AWS::XRay::ResourcePolicy": "1eefd10339461fe6",
    "AWS::XRay::SamplingRule": "0aa60791e2ac7d03",
    "Module": "baab0e57f109673a",
}



"""
Copyright 2019 Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from __future__ import annotations

import logging
from typing import Any, Iterator

from cfnlint.config import ConfigMixIn
from cfnlint.decode import decode
from cfnlint.exceptions import InvalidRegionException
from cfnlint.helpers import REGIONS
from cfnlint.rules import Match, Rules
from cfnlint.rules.errors import ParseError, TransformError
from cfnlint.template.template import Template

LOGGER = logging.getLogger(__name__)


def _dedup(matches: Iterator[Match]) -> Iterator[Match]:
    """
    Deduplicate a sequence of matches.

    Args:
        matches (Iterator[Match]): The sequence of matches to be deduplicated.

    Yields:
        Match: The unique matches from the input sequence.
    """
    seen: list[Match] = []
    for match in matches:
        if match not in seen:
            seen.append(match)
            yield match


def _check_metadata_directives(
    matches: Iterator[Match], cfn: Template, config: ConfigMixIn
) -> Iterator[Match]:
    """
    Filter matches based on metadata directives in the template.

    Args:
        matches (Iterator[Match]): The sequence of matches to be filtered.

    Yields:
        Match: The matches that are not suppressed by metadata directives.
    """
    directives = cfn.get_directives()

    for match in matches:
        if match.rule.id not in directives:
            yield match
        else:
            for mandatory_check in config.mandatory_checks:
                if match.rule.id.startswith(mandatory_check):
                    yield match
                    break
            else:
                path = getattr(match, "path", None)
                if path:
                    if len(path) >= 2:
                        if path[0] != "Resources":
                            yield match
                            continue
                        if path[1] not in directives[match.rule.id]:
                            yield match
                    else:
                        yield match


def _run_template_per_config(
    cfn: Template, config: ConfigMixIn, rules: Rules
) -> Iterator[Match]:
    LOGGER.info("Run scan of template %s", cfn.filename)
    if not set(config.regions).issubset(set(REGIONS)):
        unsupported_regions = list(set(config.regions).difference(set(REGIONS)))
        raise InvalidRegionException(
            f"Regions {unsupported_regions!r} are unsupported. "
            f"Supported regions are {REGIONS!r}",
            32,
        )

    matches = cfn.transform()
    if matches:
        if rules.is_rule_enabled(TransformError(), config):
            yield from iter(matches)
        return

    if cfn.template is not None:
        if config.build_graph:
            cfn.build_graph()
        yield from _check_metadata_directives(
            rules.run(filename=cfn.filename, cfn=cfn, config=config),
            cfn=cfn,
            config=config,
        )


def _run_template(
    filename: str | None, template: Any, config: ConfigMixIn, rules: Rules
) -> Iterator[Match]:
    if not isinstance(template, dict):
        # pylint: disable=import-outside-toplevel
        from cfnlint.rules.errors import ParseError

        # Template isn't a dict which means nearly nothing will work
        yield Match.create(
            filename=filename or "",
            rule=ParseError(),
            message="Template needs to be an object.",
        )

        return

    config.set_template_args(template)
    cfn = Template(filename, template, config.regions, config.parameters)
    yield from _dedup(_run_template_per_config(cfn, config, rules))


def run_template_by_file_path(
    filename: str | None, config: ConfigMixIn, rules: Rules, ignore_bad_template: bool
) -> Iterator[Match]:
    """
    Runs a set of rules against a CloudFormation template.

    Attributes:
        config (ConfigMixIn): The configuration object containing
        settings for the template scan.
        cfn (Template): The CloudFormation template object.
        rules (Rules): The set of rules to be applied to the template.
    """

    (template, matches) = decode(filename)  # type: ignore
    if matches:
        if ignore_bad_template or any(
            "E0000".startswith(x) for x in config.ignore_checks
        ):
            matches = [match for match in matches if match.rule.id != "E0000"]

        yield from iter(matches)
        return
    yield from _run_template(filename, template, config, rules)


def run_template_by_data(
    template: dict[str, Any], config: ConfigMixIn, rules: Rules
) -> Iterator[Match]:
    """
    Runs a set of rules against a CloudFormation template.

    Attributes:
        config (ConfigMixIn): The configuration object containing
        settings for the template scan.
        cfn (Template): The CloudFormation template object.
        rules (Rules): The set of rules to be applied to the template.
    """

    yield from _run_template(None, template, config, rules)


def run_template_by_pipe(config: ConfigMixIn, rules: Rules) -> Iterator[Match]:
    """
    Runs a set of rules against a CloudFormation template.

    Attributes:
        config (ConfigMixIn): The configuration object containing
        settings for the template scan.
        cfn (Template): The CloudFormation template object.
        rules (Rules): The set of rules to be applied to the template.
    """

    (template, matches) = decode(None)  # type: ignore
    if matches:
        yield from iter(matches)
        return
    yield from run_template_by_data(template, config, rules)  # type: ignore


def run_template_by_file_paths(config: ConfigMixIn, rules: Rules) -> Iterator[Match]:
    """
    Validate the specified filenames and yield any matches found.

    This function processes each filename in the provided sequence, decoding the
    template and validating it against the configured rules. Any matches found
    are yielded as an iterator.

    Args:
        filenames (Sequence[str | None]): The sequence of filenames to be validated.

    Yields:
        Match: The matches found during the validation process.

    Raises:
        None: This function does not raise any exceptions.
    """
    ignore_bad_template: bool = False
    if config.ignore_bad_template:
        ignore_bad_template = True
    else:
        # There is no collection at this point so we need to handle this
        # check directly
        if not ParseError().is_enabled(
            include_experimental=False,
            ignore_rules=config.ignore_checks,
            include_rules=config.include_checks,
            mandatory_rules=config.mandatory_checks,
        ):
            ignore_bad_template = True
    for filename in config.templates:
        yield from run_template_by_file_path(
            filename, config, rules, ignore_bad_template
        )



"""
Copyright 2019 Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

__all__ = [
    "main",
    "Runner",
    "run_template_by_data",
    "run_template_by_file_path",
    "expand_deployment_files",
    "CfnLintExitException",
    "InvalidRegionException",
    "UnexpectedRuleException",
]

from cfnlint.exceptions import (
    CfnLintExitException,
    InvalidRegionException,
    UnexpectedRuleException,
)
from cfnlint.runner.cli import Runner, main
from cfnlint.runner.deployment_file import expand_deployment_files
from cfnlint.runner.template import run_template_by_data, run_template_by_file_path



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0

# Code is taken from jsonschema package and adapted CloudFormation use
# https://github.com/yaml/pyyaml/blob/a2d19c0234866dc9d4d55abf3009699c258bb72f/lib/yaml/scanner.py#L46
"""

from __future__ import annotations

import fileinput
import logging
import sys

from yaml import MappingNode, SafeLoader, ScalarNode, SequenceNode

try:
    from yaml import CSafeLoader

    FastLoader = CSafeLoader  # type: ignore
except ImportError:
    FastLoader = SafeLoader  # type: ignore

from yaml.constructor import ConstructorError, SafeConstructor
from yaml.parser import ParserError
from yaml.scanner import ScannerError

from cfnlint.decode.mark import Mark
from cfnlint.decode.node import dict_node, list_node, str_node
from cfnlint.rules import Match
from cfnlint.rules.errors import ParseError

UNCONVERTED_SUFFIXES = ["Ref", "Condition"]
FN_PREFIX = "Fn::"

LOGGER = logging.getLogger(__name__)


class CfnParseError(ConstructorError):
    """
    Error thrown when the template contains Cfn Error
    """

    def __init__(self, filename, errors):
        if isinstance(errors, Match):
            errors = [errors]

        # Call the base class constructor with the parameters it needs
        super().__init__(errors[0].message)

        # Now for your custom code...
        self.filename = filename
        self.matches = errors


def build_match(filename, message, line_number, column_number, key):
    return Match.create(
        message=message,
        filename=filename,
        rule=ParseError(),
        linenumber=line_number + 1,
        columnnumber=column_number + 1,
        linenumberend=line_number + 1,
        columnnumberend=column_number + 1 + len(key),
    )


class NodeConstructor(SafeConstructor):
    """
    Node Constructors for loading different types in Yaml
    """

    def __init__(self, filename):
        # Call the base class constructor
        super().__init__()
        self.filename = filename

    def flatten_mapping(self, node):
        # Rewrote to handle merging and overwriting keys
        if any(key_node.tag == "tag:yaml.org,2002:merge" for key_node, _ in node.value):
            super().flatten_mapping(node)
            setattr(node, "using_merge", True)
        super().flatten_mapping(node)

    # To support lazy loading, the original constructors first yield
    # an empty object, then fill them in when iterated. Due to
    # laziness we omit this behaviour (and will only do "deep
    # construction") by first exhausting iterators, then yielding
    # copies.
    def construct_yaml_map(self, node):
        # Check for duplicate keys on the current level, this is not desirable
        # because a dict does not support this. It overwrites it with the last
        # occurance, which can give unexpected results
        mapping = {}
        self.flatten_mapping(node)
        matches = []
        for key_node, value_node in node.value:
            key = self.construct_object(key_node, False)
            value = self.construct_object(value_node, False)

            if key is None:
                raise CfnParseError(
                    self.filename,
                    [
                        build_match(
                            filename=self.filename,
                            message=(
                                f"Null key {key_node.value!r} not supported "
                                f"(line {key_node.start_mark.line + 1})"
                            ),
                            line_number=key_node.start_mark.line,
                            column_number=key_node.start_mark.column,
                            key=key_node.value,
                        ),
                    ],
                )
            if not getattr(node, "using_merge", False):
                for key_dup in mapping:
                    if key_dup == key:
                        if matches:
                            matches.append(
                                build_match(
                                    filename=self.filename,
                                    message=(
                                        f"Duplicate found {key!r} (line"
                                        f" {key_node.start_mark.line + 1})"
                                    ),
                                    line_number=key_node.start_mark.line,
                                    column_number=key_node.start_mark.column,
                                    key=key,
                                )
                            )
                        else:
                            matches.extend(
                                [
                                    build_match(
                                        filename=self.filename,
                                        message=(
                                            f"Duplicate found {key!r} (line"
                                            f" {key_dup.start_mark.line + 1})"
                                        ),
                                        line_number=key_dup.start_mark.line,
                                        column_number=key_dup.start_mark.column,
                                        key=key,
                                    ),
                                    build_match(
                                        filename=self.filename,
                                        message=(
                                            f"Duplicate found {key!r} (line"
                                            f" {key_node.start_mark.line + 1})"
                                        ),
                                        line_number=key_node.start_mark.line,
                                        column_number=key_node.start_mark.column,
                                        key=key,
                                    ),
                                ],
                            )
            try:
                mapping[key] = value
            except Exception as exc:
                raise CfnParseError(
                    self.filename,
                    [
                        build_match(
                            filename=self.filename,
                            message=(
                                f'Unhashable type "{key}" (line'
                                f" {key.start_mark.line + 1})"
                            ),
                            line_number=key.start_mark.line,
                            column_number=key.start_mark.column,
                            key=key,
                        ),
                    ],
                ) from exc

        if matches:
            raise CfnParseError(
                self.filename,
                matches,
            )

        (obj,) = SafeConstructor.construct_yaml_map(self, node)

        using_merge = False if not hasattr(node, "using_merge") else node.using_merge
        return dict_node(obj, node.start_mark, node.end_mark, using_merge)

    def construct_yaml_str(self, node):
        obj = SafeConstructor.construct_yaml_str(self, node)
        assert isinstance(obj, (str))
        return str_node(obj, node.start_mark, node.end_mark)

    def construct_yaml_seq(self, node):
        (obj,) = SafeConstructor.construct_yaml_seq(self, node)
        assert isinstance(obj, list)
        return list_node(obj, node.start_mark, node.end_mark)

    def construct_getatt(self, node):
        """
        Reconstruct !GetAtt into a list
        """
        if isinstance(node.value, (str)):
            return list_node(node.value.split(".", 1), node.start_mark, node.end_mark)
        if isinstance(node.value, list):
            if any(isinstance(value, tuple) for value in node.value):
                return dict_node(
                    self.construct_object(
                        MappingNode(
                            tag="tag:yaml.org,2002:map",
                            value=node.value,
                            start_mark=node.start_mark,
                            end_mark=node.end_mark,
                        ),
                        deep=True,
                    )
                )
            else:
                return list_node(
                    [self.construct_object(child, deep=True) for child in node.value],
                    node.start_mark,
                    node.end_mark,
                )
        raise ValueError(f"Unexpected GetAtt format: {type(node.value)}")


NodeConstructor.add_constructor(  # type: ignore
    "tag:yaml.org,2002:map", NodeConstructor.construct_yaml_map
)

NodeConstructor.add_constructor(  # type: ignore
    "tag:yaml.org,2002:str", NodeConstructor.construct_yaml_str
)

NodeConstructor.add_constructor(  # type: ignore
    "tag:yaml.org,2002:seq", NodeConstructor.construct_yaml_seq
)


class CfnFullLoader(FastLoader, NodeConstructor):
    """
    Custom FullLoader that integrates NodeConstructor functionality
    """

    def __init__(self, stream, filename):
        FastLoader.__init__(self, stream)
        NodeConstructor.__init__(self, filename)


# Register NodeConstructor methods with CfnFullLoader
CfnFullLoader.add_constructor(  # type: ignore[type-var]
    "tag:yaml.org,2002:map", NodeConstructor.construct_yaml_map
)
CfnFullLoader.add_constructor(  # type: ignore[type-var]
    "tag:yaml.org,2002:str", NodeConstructor.construct_yaml_str
)
CfnFullLoader.add_constructor(  # type: ignore[type-var]
    "tag:yaml.org,2002:seq", NodeConstructor.construct_yaml_seq
)


def multi_constructor(loader: CfnFullLoader, tag_suffix, node):
    """
    Deal with !Ref style function format
    """

    if tag_suffix not in UNCONVERTED_SUFFIXES:
        tag_suffix = f"{FN_PREFIX}{tag_suffix}"

    if tag_suffix == "Fn::GetAtt":
        return dict_node(
            {tag_suffix: loader.construct_getatt(node)}, node.start_mark, node.end_mark
        )
    elif isinstance(node, ScalarNode):
        return dict_node(
            {tag_suffix: loader.construct_scalar(node)}, node.start_mark, node.end_mark
        )
    elif isinstance(node, SequenceNode):
        return dict_node(
            {tag_suffix: loader.construct_sequence(node, True)},
            node.start_mark,
            node.end_mark,
        )
    elif isinstance(node, MappingNode):
        return dict_node(
            {tag_suffix: loader.construct_mapping(node, True)},
            node.start_mark,
            node.end_mark,
        )

    raise Exception(f"Bad tag: !{tag_suffix}")


def loads(yaml_string, fname=None):
    """
    Load the given YAML string
    """
    try:
        loader = CfnFullLoader(yaml_string, fname)
        loader.add_multi_constructor("!", multi_constructor)

        template = loader.get_single_data()
        # Convert an empty file to an empty dict
        if template is None:
            template = dict_node({}, Mark(0, 0), Mark(0, 0))
        return template
    except CfnParseError:
        raise
    except (ScannerError, ParserError):
        # Let YAML parsing errors bubble up so decode.py can handle JSON fallback
        raise
    except Exception as exc:
        raise CfnParseError(
            fname,
            [
                build_match(
                    filename=fname,
                    message=f"Template could not be parsed: {str(exc)}",
                    line_number=0,
                    column_number=0,
                    key="",
                )
            ],
        ) from exc


def load(filename):
    """
    Load the given YAML file
    """

    content = ""

    if (filename is None) and (not sys.stdin.isatty()):
        filename = "-"  # no filename provided, it's stdin
        fileinput_args = {"files": filename}
        if sys.version_info.major <= 3 and sys.version_info.minor >= 10:
            fileinput_args["encoding"] = "utf-8"
        with fileinput.input(**fileinput_args) as f:
            for line in f:
                content = content + line
    else:
        with open(filename, encoding="utf-8") as fp:
            content = fp.read()

    return loads(content, filename)



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from __future__ import annotations

import os
import tempfile
from pathlib import Path
from unittest import TestCase

from cfnlint import lint, lint_all, lint_by_config, lint_file
from cfnlint.config import ManualArgs
from cfnlint.core import get_rules
from cfnlint.helpers import REGIONS
from cfnlint.rules.errors import ConfigError


class TestLint(TestCase):
    def helper_lint_string_from_file(
        self, filename=None, config: ManualArgs | None = None
    ):
        with open(filename, "r") as f:
            return lint(f.read(), config=config)

    def helper_lint_string_from_file_all(self, filename):
        with open(filename, "r") as f:
            return lint_all(f.read())

    def test_noecho_yaml_template(self):
        filename = "test/fixtures/templates/bad/noecho.yaml"
        matches = self.helper_lint_string_from_file(
            filename=filename,
            config=ManualArgs(regions=["us-east-1", "us-west-2", "eu-west-1"]),
        )
        self.assertEqual(
            ["W2010", "W2010"],
            [match.rule.id for match in matches],
            f"Got matches: {matches!r}",
        )

    def test_noecho_yaml_template_warnings_ignored(self):
        filename = "test/fixtures/templates/bad/noecho.yaml"
        matches = self.helper_lint_string_from_file(
            filename=filename,
            config=ManualArgs(
                ignore_checks=["W", "I"],
            ),
        )
        self.assertListEqual([], matches, f"Got matches: {matches!r}")

    def test_duplicate_json_template(self):
        filename = "test/fixtures/templates/bad/duplicate.json"
        matches = self.helper_lint_string_from_file(
            filename=filename,
            config=ManualArgs(
                regions=["us-east-1", "us-west-2", "eu-west-1"],
            ),
        )
        self.assertEqual(
            ["E0000", "E0000", "E0000"],
            [match.rule.id for match in matches],
            f"Got matches: {matches!r}",
        )

    def test_invalid_yaml_template(self):
        filename = "test/fixtures/templates/bad/core/config_invalid_yaml.yaml"
        matches = self.helper_lint_string_from_file(
            filename=filename,
            config=ManualArgs(regions=["us-east-1", "us-west-2", "eu-west-1"]),
        )
        self.assertEqual(
            ["E0000"], [match.rule.id for match in matches], f"Got matches: {matches!r}"
        )

    def test_invalid_yaml_template_lint_all(self):
        filename = "test/fixtures/templates/bad/core/config_invalid_yaml.yaml"
        matches = self.helper_lint_string_from_file_all(filename=filename)
        self.assertEqual(
            ["E0000"], [match.rule.id for match in matches], f"Got matches: {matches!r}"
        )

    def test_invalid_json_template(self):
        filename = "test/fixtures/templates/bad/core/config_invalid_json.json"
        matches = self.helper_lint_string_from_file(
            filename=filename,
            config=ManualArgs(regions=["us-east-1", "us-west-2", "eu-west-1"]),
        )
        self.assertEqual(
            ["E0000"], [match.rule.id for match in matches], f"Got matches: {matches!r}"
        )

    def test_invalid_json_template_lint_all(self):
        filename = "test/fixtures/templates/bad/core/config_invalid_json.json"
        matches = self.helper_lint_string_from_file_all(filename=filename)
        self.assertEqual(["E0000"], [match.rule.id for match in matches])

    def test_issues_template(self):
        filename = "test/fixtures/templates/bad/issues.yaml"
        matches = self.helper_lint_string_from_file(
            filename=filename,
            config=ManualArgs(regions=["us-east-1", "us-west-2", "eu-west-1"]),
        )
        self.assertEqual(
            ["E1020"], [match.rule.id for match in matches], f"Got matches: {matches!r}"
        )

    def test_sam_template(self):
        filename = "test/fixtures/templates/good/transform/list_transform_many.yaml"
        matches = self.helper_lint_string_from_file(filename)
        self.assertEqual([], matches, f"Got matches: {matches!r}")


class TestV0Usage(TestCase):
    def helper_lint_string_from_file(
        self,
        filename,
        rules=get_rules([], [], ["I", "W", "E"], include_experimental=True),
        regions=REGIONS,
    ):
        with open(filename, "r") as f:
            return lint(f.read(), rules, regions)

    def test_noecho_yaml_template(self):
        filename = "test/fixtures/templates/bad/noecho.yaml"
        matches = self.helper_lint_string_from_file(filename)
        self.assertEqual(
            ["W2010", "W2010"],
            [match.rule.id for match in matches],
            f"Got matches: {matches!r}",
        )

    def test_noecho_yaml_template_warnings_ignored(self):
        filename = "test/fixtures/templates/bad/noecho.yaml"
        rules = get_rules([], ["W", "I"], [])
        matches = self.helper_lint_string_from_file(filename, rules=rules)
        self.assertEqual([], matches, f"Got {matches!r}")

    def test_duplicate_json_template(self):
        filename = "test/fixtures/templates/bad/duplicate.json"
        matches = self.helper_lint_string_from_file(filename)
        self.assertEqual(
            ["E0000", "E0000", "E0000"],
            [match.rule.id for match in matches],
            f"Got matches: {matches!r}",
        )

    def test_invalid_yaml_template(self):
        filename = "test/fixtures/templates/bad/core/config_invalid_yaml.yaml"
        matches = self.helper_lint_string_from_file(filename)
        self.assertEqual(
            ["E0000"], [match.rule.id for match in matches], f"Got matches: {matches!r}"
        )

    def test_invalid_json_template(self):
        filename = "test/fixtures/templates/bad/core/config_invalid_json.json"
        matches = self.helper_lint_string_from_file(filename)
        self.assertEqual(
            ["E0000"], [match.rule.id for match in matches], f"Got matches: {matches!r}"
        )

    def test_issues_template(self):
        filename = "test/fixtures/templates/bad/issues.yaml"
        matches = self.helper_lint_string_from_file(filename)
        self.assertEqual(
            ["E1020"], [match.rule.id for match in matches], f"Got matches: {matches!r}"
        )

    def test_sam_template(self):
        filename = "test/fixtures/templates/good/transform/list_transform_many.yaml"
        matches = self.helper_lint_string_from_file(filename)
        self.assertEqual([], matches, f"Got matches: {matches!r}")


class TestLintFile(TestCase):
    """Test the lint_file API function"""

    def test_nonexistent_file(self):
        """Test linting a file that doesn't exist"""
        matches = lint_file(Path("nonexistent_file.yaml"))
        self.assertEqual(1, len(matches))
        self.assertEqual("E0000", matches[0].rule.id)
        self.assertIn("Template file not found", matches[0].message)

    def test_noecho_yaml_template(self):
        """Test linting a template with NoEcho issues"""
        filename = Path("test/fixtures/templates/bad/noecho.yaml")
        matches = lint_file(
            filename,
            config=ManualArgs(regions=["us-east-1", "us-west-2", "eu-west-1"]),
        )
        self.assertEqual(
            ["W2010", "W2010"],
            [match.rule.id for match in matches],
            f"Got matches: {matches!r}",
        )

    def test_noecho_yaml_template_warnings_ignored(self):
        """Test linting with warnings ignored"""
        filename = Path("test/fixtures/templates/bad/noecho.yaml")
        matches = lint_file(
            filename,
            config=ManualArgs(
                ignore_checks=["W", "I"],
            ),
        )
        self.assertListEqual([], matches, f"Got matches: {matches!r}")

    def test_duplicate_json_template(self):
        """Test linting a template with duplicate keys"""
        filename = Path("test/fixtures/templates/bad/duplicate.json")
        matches = lint_file(
            filename,
            config=ManualArgs(
                regions=["us-east-1", "us-west-2", "eu-west-1"],
            ),
        )
        self.assertEqual(
            ["E0000", "E0000", "E0000"],
            [match.rule.id for match in matches],
            f"Got matches: {matches!r}",
        )

    def test_invalid_yaml_template(self):
        """Test linting an invalid YAML template"""
        filename = Path("test/fixtures/templates/bad/core/config_invalid_yaml.yaml")
        matches = lint_file(
            filename,
            config=ManualArgs(regions=["us-east-1", "us-west-2", "eu-west-1"]),
        )
        self.assertEqual(
            ["E0000"], [match.rule.id for match in matches], f"Got matches: {matches!r}"
        )

    def test_invalid_json_template(self):
        """Test linting an invalid JSON template"""
        filename = Path("test/fixtures/templates/bad/core/config_invalid_json.json")
        matches = lint_file(
            filename,
            config=ManualArgs(regions=["us-east-1", "us-west-2", "eu-west-1"]),
        )
        self.assertEqual(
            ["E0000"], [match.rule.id for match in matches], f"Got matches: {matches!r}"
        )

    def test_issues_template(self):
        """Test linting a template with issues"""
        filename = Path("test/fixtures/templates/bad/issues.yaml")
        matches = lint_file(
            filename,
            config=ManualArgs(regions=["us-east-1", "us-west-2", "eu-west-1"]),
        )
        self.assertEqual(
            ["E1020"], [match.rule.id for match in matches], f"Got matches: {matches!r}"
        )

    def test_sam_template(self):
        """Test linting a SAM template"""
        filename = Path(
            "test/fixtures/templates/good/transform/list_transform_many.yaml"
        )
        matches = lint_file(filename)
        self.assertEqual([], matches, f"Got matches: {matches!r}")

    def test_empty_file(self):
        """Test linting an empty file"""
        with tempfile.NamedTemporaryFile(delete=False) as temp_file:
            temp_file.write(b"")
            temp_file_path = Path(temp_file.name)

        try:
            matches = lint_file(temp_file_path)
            self.assertEqual(1, len(matches))
            self.assertEqual("E1001", matches[0].rule.id)
        finally:
            os.unlink(temp_file_path)

    def test_good_template(self):
        """Test linting a good template"""
        filename = Path("test/fixtures/templates/good/generic.yaml")
        matches = lint_file(filename)
        self.assertEqual([], matches, f"Got matches: {matches!r}")


class TestLintByConfig(TestCase):
    """Test lint_by_config function"""

    def test_lint_by_config_with_invalid_config(self):
        """Test lint_by_config returns ConfigError for invalid configuration"""
        # Test with no templates or deployment files
        result = lint_by_config(ManualArgs())

        self.assertEqual(len(result), 1)
        self.assertIsInstance(result[0].rule, ConfigError)
        self.assertIn("No templates or deployment files specified", result[0].message)

    def test_lint_by_config_with_conflicting_deployment_files(self):
        """Test lint_by_config returns ConfigError for conflicting deployment files"""
        result = lint_by_config(
            ManualArgs(
                deployment_files=["deployment.yaml"], templates=["template.yaml"]
            )
        )

        self.assertEqual(len(result), 1)
        self.assertIsInstance(result[0].rule, ConfigError)
        self.assertIn("Deployment files cannot be used with", result[0].message)

    def test_lint_by_config_with_conflicting_parameters(self):
        """Test lint_by_config returns ConfigError for conflicting parameters"""
        result = lint_by_config(
            ManualArgs(
                templates=["template.yaml"],
                parameters={"Key": "Value"},
                parameter_files=["params.json"],
            )
        )

        self.assertEqual(len(result), 1)
        self.assertIsInstance(result[0].rule, ConfigError)
        self.assertIn(
            "Cannot specify both --parameters and --parameter-files", result[0].message
        )

    def test_lint_by_config_with_multiple_templates_and_parameters(self):
        """Test lint_by_config returns ConfigError for multiple templates
        with parameters
        """
        result = lint_by_config(
            ManualArgs(
                templates=["template1.yaml", "template2.yaml"],
                parameters={"Key": "Value"},
            )
        )

        self.assertEqual(len(result), 1)
        self.assertIsInstance(result[0].rule, ConfigError)
        self.assertIn(
            "Parameters can only be used with a single template", result[0].message
        )

    def test_lint_by_config_with_valid_config_runs_successfully(self):
        """Test lint_by_config runs successfully with valid config and calls
        runner.run()"""
        # Create a simple valid template file for testing
        import os
        import tempfile

        with tempfile.NamedTemporaryFile(mode="w", suffix=".yaml", delete=False) as f:
            f.write(
                """
AWSTemplateFormatVersion: '2010-09-09'
Resources:
  TestResource:
    Type: AWS::S3::Bucket
"""
            )
            temp_file = f.name

        try:
            result = lint_by_config(ManualArgs(templates=[temp_file]))

            # Should return a list (may be empty or contain linting errors,
            # but should not be a config error)
            self.assertIsInstance(result, list)
            # If there are any results, they should not be ConfigError
            for match in result:
                self.assertNotIsInstance(match.rule, ConfigError)

        finally:
            # Clean up the temporary file
            os.unlink(temp_file)



"""
Copyright 2019 Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from __future__ import annotations

import filecmp
import fnmatch
import json
import logging
import multiprocessing
import os
import re
import shutil
import sys
import zipfile
from functools import lru_cache
from pathlib import Path
from typing import TYPE_CHECKING, Any, Dict, Iterator, Sequence

import jsonpatch
import jsonpointer

from cfnlint.helpers import (
    REGION_PRIMARY,
    REGIONS,
    ToPy,
    get_url_retrieve,
    load_resource,
    url_has_newer_version,
)
from cfnlint.schema._exceptions import ResourceNotFoundError
from cfnlint.schema._getatts import AttributeDict
from cfnlint.schema._schema import Schema

if TYPE_CHECKING:
    from cfnlint.schema._patch import SchemaPatch

LOGGER = logging.getLogger(__name__)


class _FileLocation:
    def __init__(self, path: list[str]):
        self.path_relative = os.path.join(
            os.path.dirname(__file__),
            "..",
            *path,
        )
        self.module = ".".join(["cfnlint"] + path[:])
        self.path = path


class ProviderSchemaManager:
    def __init__(self) -> None:
        self._root = _FileLocation(
            [
                "data",
                "schemas",
                "providers",
            ]
        )
        self._patches = _FileLocation(
            [
                "data",
                "schemas",
                "patches",
            ]
        )
        self._region_primary = ToPy(REGION_PRIMARY)
        self._registry_schemas: dict[str, Schema] = {}
        self._provider_schema_modules: dict[str, Any] = {}
        self.reset()

    def reset(self) -> None:
        """
        Reset's the cache so specs can be reloaded.
        Important function when processing many templates
        and using spec patching
        """
        self._schemas: dict[str, dict[str, Schema]] = {}
        for region in REGIONS:
            self._schemas[region] = {}
        self._removed_types: list[str] = []
        self.get_resource_schema.cache_clear()
        self.get_resource_types.cache_clear()
        self.get_type_getatts.cache_clear()

    def load_registry_schemas(self, path: str) -> None:
        """Load extra registry schemas from a directory

        Args:
            path (str): the directory to load schema files from
        Returns:
            None: None
        """
        for dirpath, _, filenames in os.walk(path):
            for filename in fnmatch.filter(filenames, "*.json"):
                with open(os.path.join(dirpath, filename), "r", encoding="utf-8") as fh:
                    schema = Schema(json.load(fh))
                    self._registry_schemas[schema.type_name] = schema

    def get_resource_schemas_by_regions(
        self, resource_type: str, regions: Sequence[str]
    ) -> Iterator[tuple[list[str], Schema]]:
        """
        Get unique schemas with their associated regions

        Args:
            resource_type (str): the :: version of the resource type
            regions (Sequence[str]): the regions in which we want schems for
        Returns:
            Iterator[tuple[list[str], Schema]]: the unique schemas with
              their associated regions
        """
        # Normalize resource type (Custom:: -> AWS::CloudFormation::CustomResource)
        resource_type = self._normalize_resource_type(resource_type)

        # Group regions by schema hash to avoid duplicate validations
        hash_to_regions: dict[str, list[str]] = {}
        hash_to_schema: dict[str, Schema] = {}

        for region in regions:
            # Get the hash for this region's schema first
            reg = ToPy(region)
            if reg.name not in self._provider_schema_modules:
                self._provider_schema_modules[reg.name] = __import__(
                    f"{self._root.module}.{reg.py}", fromlist=[""]
                )
            schema_hash = self._provider_schema_modules[reg.name].types.get(
                resource_type
            )

            if not schema_hash:
                continue

            # Only load schema if we haven't seen this hash before
            if schema_hash not in hash_to_regions:
                try:
                    schema = self.get_resource_schema(region, resource_type)
                    hash_to_regions[schema_hash] = []
                    hash_to_schema[schema_hash] = schema
                except ResourceNotFoundError:
                    continue

            hash_to_regions[schema_hash].append(region)

        # Yield each unique schema with its regions
        for schema_hash, region_list in hash_to_regions.items():
            yield region_list, hash_to_schema[schema_hash]

    def _normalize_resource_type(self, resource_type: str) -> str:
        """
        Normalize the resource type to the correct format

        Args:
            resource_type (str): the :: version of the resource type
        Returns:
            str: the normalized resource type
        """
        if resource_type.startswith("Custom::"):
            resource_type = "AWS::CloudFormation::CustomResource"
        if resource_type.endswith("::MODULE"):
            resource_type = "Module"

        return resource_type

    @lru_cache(maxsize=None)
    def get_resource_schema(self, region: str, resource_type: str) -> Schema:
        """Get the provider resource schema and cache it to speed up future lookups

        Args:
            region (str): the region in which do get the provider schema for
            resource_type (str): the :: version of the resource type
        Returns:
            dict: returns the schema
        """
        if resource_type not in self._registry_schemas:
            resource_type = self._normalize_resource_type(resource_type)

        if resource_type in self._removed_types:
            raise ResourceNotFoundError(resource_type, region)

        reg = ToPy(region)
        rt = ToPy(resource_type)

        schema = self._schemas[reg.name].get(rt.name)
        if schema is not None:
            return schema

        # dynamically import the region module as needed
        if reg.name not in self._provider_schema_modules:
            self._provider_schema_modules[reg.name] = __import__(
                f"{self._root.module}.{reg.py}", fromlist=[""]
            )

        # check cfn-lint provided schemas
        if rt.name in self._registry_schemas:
            self._schemas[reg.name][rt.name] = self._registry_schemas[rt.name]
            return self._schemas[reg.name][rt.name]

        # load the schema from hash-based storage
        region_module = self._provider_schema_modules[reg.name]
        schema_hash = region_module.types.get(resource_type)

        if not schema_hash:
            raise ResourceNotFoundError(rt.name, region)

        try:
            # Load from resources directory using hash
            resources_module = __import__(
                "cfnlint.data.schemas.resources", fromlist=[""]
            )
            self._schemas[reg.name][rt.name] = Schema(
                load_resource(
                    resources_module,
                    filename=f"{schema_hash}.json",
                )
            )
            return self._schemas[reg.name][rt.name]
        except Exception as e:
            raise ResourceNotFoundError(rt.name, region) from e

    @lru_cache(maxsize=None)
    def get_resource_types(self, region: str) -> list[str]:
        """Get the resource types for a region

        Args:
            region (str): the region in which to get the resource types for
        Returns:
            list[str]: returns a list of resource types
        """
        reg = ToPy(region)

        if reg.name not in self._provider_schema_modules:
            self._provider_schema_modules[reg.name] = __import__(
                f"{self._root.module}.{reg.py}", fromlist=[""]
            )

        resource_types: list[str] = []
        resource_types.extend(
            rt
            for rt in self._provider_schema_modules[reg.name].types.keys()
            if rt not in self._removed_types
        )
        resource_types.extend(list(self._registry_schemas.keys()))

        return resource_types

    def update(self, force: bool) -> int:
        """Update every regions provider schemas

        Args:
            force (bool): force the schemas to be downloaded
        Returns:
            int: exit code (0=success, 1=partial failure, 2=complete failure)
        """
        import hashlib
        from pathlib import Path

        # Download all regions in parallel
        downloaded_regions = {}
        failed_regions = []

        # pylint: disable=not-context-manager
        with multiprocessing.Pool() as pool:
            results = pool.starmap(
                self._download_region_schemas, [(region, force) for region in REGIONS]
            )

        for region, result in zip(REGIONS, results):
            if result is None:
                failed_regions.append(region)
            else:
                downloaded_regions[region] = result

        if not downloaded_regions:
            LOGGER.error("All regions failed to download")
            return 2

        # Build global hash map
        hash_to_schema = {}
        region_mappings: dict[str, dict[str, str]] = {}

        for region, schemas in downloaded_regions.items():
            region_mappings[region] = {}
            for resource_type, schema_content in schemas.items():
                schema_hash = hashlib.sha256(
                    json.dumps(schema_content, sort_keys=True).encode()
                ).hexdigest()[:16]

                hash_to_schema[schema_hash] = schema_content
                region_mappings[region][resource_type] = schema_hash

        # Write unique schemas
        resources_dir = Path(self._root.path_relative).parent / "resources"
        resources_dir.mkdir(exist_ok=True)

        for schema_hash, schema_content in hash_to_schema.items():
            with open(
                resources_dir / f"{schema_hash}.json", "w", encoding="utf-8"
            ) as f:
                json.dump(
                    schema_content, f, indent=1, separators=(",", ": "), sort_keys=True
                )
                f.write("\n")

        # Write region mapping files
        for region, type_map in region_mappings.items():
            self._write_region_file(region, type_map)

        # Handle ISO regions by referencing us-east-1
        if "us-east-1" in region_mappings:
            iso_handled = []
            for region in failed_regions:
                if "iso" in region or region.startswith("eusc"):
                    self._write_region_file(region, region_mappings["us-east-1"])
                    iso_handled.append(region)
            # Remove ISO regions from failed list since they're expected to fail
            for region in iso_handled:
                failed_regions.remove(region)

        # Cleanup orphaned schemas
        self._cleanup_orphaned_schemas(region_mappings, resources_dir)

        if failed_regions:
            LOGGER.warning(
                f"Failed regions (kept existing): {', '.join(failed_regions)}"
            )
            return 1

        return 0

    def _download_region_schemas(
        self, region: str, force: bool = False
    ) -> dict[str, dict] | None:
        """Download schemas for a single region

        Args:
            region: Region to download
            force: Force download even if cached
        Returns:
            Dict mapping resource type to schema content, or None on failure
        """

        suffix = ".cn" if region in ["cn-north-1", "cn-northwest-1"] else ""
        url = f"https://schema.cloudformation.{region}.amazonaws.com{suffix}/CloudformationSchema.zip"

        multiprocessing_logger = multiprocessing.log_to_stderr()
        multiprocessing_logger.debug(f"Downloading {url}")

        # Check if update needed
        if not (url_has_newer_version(url) or force):
            return None

        try:
            filehandle = get_url_retrieve(url, caching=True)
            schemas = {}

            with zipfile.ZipFile(filehandle, "r") as zip_ref:
                for filename in zip_ref.namelist():
                    if filename.endswith(".json"):
                        with zip_ref.open(filename) as f:
                            spec = json.load(f)
                            # Apply patches and cleanup
                            if "handlers" in spec:
                                del spec["handlers"]
                            if "tagging" in spec and "permissions" in spec.get(
                                "tagging", {}
                            ):
                                del spec["tagging"]["permissions"]
                            spec = self._remove_descriptions(spec)
                            spec = self._patch_provider_schema(spec, filename, "all")
                            spec = self._patch_provider_schema(
                                spec, filename, region=ToPy(region).py
                            )

                            schemas[spec["typeName"]] = spec

            # Add synthetic types
            schemas["AWS::CDK::Metadata"] = {"typeName": "AWS::CDK::Metadata"}
            schemas["Module"] = {
                "additionalProperties": True,
                "type": "object",
                "typeName": "Module",
            }

            return schemas

        except Exception as e:
            LOGGER.warning(f"Failed downloading {region}: {e}")
            return None

    def _write_region_file(self, region: str, type_map: dict[str, str]) -> None:
        """Write region .py file with type to hash mapping

        Args:
            region: Region name
            type_map: Dict mapping resource type to schema hash
        """
        from pathlib import Path

        reg = ToPy(region)
        region_file = Path(self._root.path_relative) / f"{reg.py}.py"

        with open(region_file, "w", encoding="utf-8") as f:
            f.write("# ruff: noqa: E501, PLR0915\n")
            f.write("from __future__ import annotations\n\n")
            f.write("types: dict[str, str] = {\n")
            for resource_type in sorted(type_map.keys()):
                schema_hash = type_map[resource_type]
                f.write(f'    "{resource_type}": "{schema_hash}",\n')
            f.write("}\n")

    def _cleanup_orphaned_schemas(self, region_mappings: dict, resources_dir) -> None:
        """Remove schema files no longer referenced by any region

        Args:
            region_mappings: Dict of region to type mappings (newly downloaded)
            resources_dir: Path to resources directory
        """
        from pathlib import Path

        # Collect all referenced hashes from newly downloaded regions
        referenced_hashes = set()
        for type_map in region_mappings.values():
            referenced_hashes.update(type_map.values())

        # Also check existing provider files for regions that weren't updated
        providers_dir = Path(self._root.path_relative)
        for provider_file in providers_dir.glob("*.py"):
            if provider_file.name == "__init__.py":
                continue
            try:
                # Import the provider module to get its types dict
                region_name = provider_file.stem
                module_name = f"cfnlint.data.schemas.providers.{region_name}"
                provider = __import__(module_name, fromlist=["types"])
                if hasattr(provider, "types"):
                    referenced_hashes.update(provider.types.values())
            except (ImportError, AttributeError):
                continue

        # Remove unreferenced files
        for schema_file in Path(resources_dir).glob("*.json"):
            schema_hash = schema_file.stem
            if schema_hash not in referenced_hashes:
                LOGGER.info(f"Removing orphaned schema: {schema_hash}")
                schema_file.unlink()

    def _remove_descriptions(self, spec: Any) -> Any:
        if isinstance(spec, dict):
            r: dict[Any, Any] = {}
            for k, v in spec.items():
                if k != "description":
                    r[k] = self._remove_descriptions(v)

            return r
        elif isinstance(spec, list):
            m: list[Any] = []
            for v in spec:
                m.append(self._remove_descriptions(v))

            return m
        else:
            return spec

    def _update_provider_schema(self, region: str, force: bool = False) -> None:
        """Update the provider schemas from the AWS websites

        Args:
            region (str): the region in which do ge the provider schema for
            force (bool): force the schemas to be downloaded
        Returns:
            None: returns when complete
        """
        # China regions in .com.cn
        suffix = ".cn" if region in ["cn-north-1", "cn-northwest-1"] else ""
        url = f"https://schema.cloudformation.{region}.amazonaws.com{suffix}/CloudformationSchema.zip"
        reg = ToPy(region)
        directory = os.path.join(f"{self._root.path_relative}/{reg.py}/")
        directory_pr = os.path.join(
            f"{self._root.path_relative}/{self._region_primary.py}/"
        )

        multiprocessing_logger = multiprocessing.log_to_stderr()

        multiprocessing_logger.debug("Downloading template %s into %s", url, directory)

        if "iso" in region or region.startswith("eusc"):
            all_types = ["AWS::CDK::Metadata", "Module"]
            cached = ["Module"]
            filenames = [
                f
                for f in os.listdir(directory_pr)
                if os.path.isfile(os.path.join(directory_pr, f)) and f != "__init__.py"
            ]
            for filename in filenames:
                with open(f"{directory_pr}{filename}", "r+", encoding="utf-8") as fh:
                    spec = json.load(fh)
                    all_types.append(spec["typeName"])
                cached.append(filename)

            with open(f"{directory}__init__.py", encoding="utf-8", mode="w") as f:
                f.write("from __future__ import annotations\n\n")
                f.write("# pylint: disable=too-many-lines\ntypes: list[str] = [\n")
                for rt in sorted(all_types):
                    f.write(f'    "{rt}",\n')
                f.write(
                    "]\n\n# pylint: disable=too-many-lines\ncached: list[str] = [\n"
                )
                for cache_file in sorted(cached):
                    f.write(f'    "{cache_file}",\n')
                f.write("]\n")

        # Check to see if we already have the latest version, and if so stop
        if not (url_has_newer_version(url) or force):
            return

        if not os.path.exists(directory):
            os.mkdir(directory)

        try:
            filehandle = get_url_retrieve(url, caching=True)
            # clean folder
            shutil.rmtree(directory)
            with zipfile.ZipFile(filehandle, "r") as zip_ref:
                zip_ref.extractall(directory)

            filenames = [
                f
                for f in os.listdir(directory)
                if os.path.isfile(os.path.join(directory, f)) and f != "__init__.py"
            ]
            # There is no schema for CDK but its an allowable type
            all_types = ["AWS::CDK::Metadata", "Module"]
            for filename in filenames:
                with open(f"{directory}{filename}", "r+", encoding="utf-8") as fh:
                    spec = json.load(fh)
                    all_types.append(spec["typeName"])

                    # Apply patches to the schema
                    spec = self._patch_provider_schema(spec, filename, "all")
                    spec = self._patch_provider_schema(
                        spec, filename, region=ToPy(region).py
                    )

                    # Write the patched schema back
                    fh.seek(0)
                    json.dump(
                        spec, fh, indent=1, separators=(",", ": "), sort_keys=True
                    )
                    fh.write("\n")
                    fh.truncate()

            # if the region is not us-east-1 compare the files to those in us-east-1
            # symlink if the files are the same
            if reg.name != self._region_primary.name:
                cached = ["Module"]
                for filename in os.listdir(directory):
                    if filename != "__init__.py":
                        try:
                            if filecmp.cmp(
                                f"{directory}{filename}",
                                f"{directory_pr}{filename}",
                            ):
                                cached.append(filename)
                                os.remove(f"{directory}{filename}")
                        except FileNotFoundError:
                            pass
                        except Exception as e:  # pylint: disable=broad-except
                            # Exceptions will typically be the file
                            # doesn't exist in primary region
                            LOGGER.info(
                                "Issuing comparing %s into %s: %s",
                                f"{directory}{filename}",
                                f"{directory_pr}{filename}",
                                e,
                            )
                with open(f"{directory}__init__.py", encoding="utf-8", mode="w") as f:
                    f.write("from __future__ import annotations\n\n")
                    f.write("# pylint: disable=too-many-lines\ntypes: list[str] = [\n")
                    for rt in sorted(all_types):
                        f.write(f'    "{rt}",\n')
                    f.write(
                        "]\n\n# pylint: disable=too-many-lines\ncached: list[str] = [\n"
                    )
                    for cache_file in sorted(cached):
                        f.write(f'    "{cache_file}",\n')
                    f.write("]\n")
            else:
                with open(f"{directory}__init__.py", encoding="utf-8", mode="w") as f:
                    f.write("from __future__ import annotations\n\n")
                    f.write("# pylint: disable=too-many-lines\ntypes: list[str] = [\n")
                    for rt in sorted(all_types):
                        f.write(f'    "{rt}",\n')
                    f.write("]\ncached: list[str] = []\n")

        except Exception as e:  # pylint: disable=broad-except
            LOGGER.info("Issuing updating schemas for %s: %s", region, e)

    def patch_schemas(self) -> None:
        """Patch schemas in the hash-based storage system"""
        resources_dir = Path(self._root.path_relative).parent / "resources"
        if not resources_dir.exists():
            LOGGER.info("Resources directory not found, skipping patching")
            return

        # Get all schema files
        schema_files = list(resources_dir.glob("*.json"))
        LOGGER.info("Patching %d schema files", len(schema_files))

        patched_count = 0
        for schema_file in schema_files:
            try:
                # Load the schema
                with open(schema_file, "r", encoding="utf-8") as f:
                    spec = json.load(f)

                original_spec = json.dumps(spec, sort_keys=True)

                # Apply standard patches
                if "handlers" in spec:
                    del spec["handlers"]
                if "tagging" in spec and "permissions" in spec.get("tagging", {}):
                    del spec["tagging"]["permissions"]

                spec = self._remove_descriptions(spec)

                # Apply provider-specific patches based on typeName
                if "typeName" in spec:
                    type_name = spec["typeName"]
                    filename = f"{type_name.lower().replace('::', '_')}.json"
                    spec = self._patch_provider_schema(spec, filename, "all")

                # Check if the schema was modified
                modified_spec = json.dumps(spec, sort_keys=True)
                if original_spec != modified_spec:
                    # Write the patched schema back to the same file
                    with open(schema_file, "w", encoding="utf-8") as f:
                        json.dump(
                            spec, f, indent=1, separators=(",", ": "), sort_keys=True
                        )
                        f.write("\n")
                    patched_count += 1
                    LOGGER.debug("Patched schema: %s", schema_file.name)

            except Exception as e:
                LOGGER.info("Error patching %s: %s", schema_file.name, e)

        LOGGER.info(
            "Patched %d out of %d schema files", patched_count, len(schema_files)
        )

    def _patch_provider_schema(
        self, content: Dict, source_filename: str, region: str
    ) -> Dict:
        """Provides the logic to patch a CloudFormation provider schema file.

        Args:
            content: A Dict representing the data that needs to be patched
            source_filename: The source filename for the JSON patches
            region: The region to apply the patch against
        Returns:
            Dict: returns the patched content
        """
        for patch_type in ["providers", "extensions"]:
            source_dir = source_filename.replace("-", "_").replace(".json", "")
            append_dir = os.path.join(
                self._patches.path_relative, patch_type, region, source_dir
            )
            for dirpath, _, filenames in os.walk(append_dir):
                filenames.sort()
                for filename in fnmatch.filter(filenames, "*.json"):
                    file_path = os.path.basename(filename)
                    module = dirpath.replace(f"{append_dir}", f"{region}").replace(
                        os.path.sep, "."
                    )
                    try:
                        jsonpatch.JsonPatch(
                            load_resource(
                                f"{self._patches.module}.{patch_type}.{module}.{source_dir}",
                                file_path,
                            )
                        ).apply(content, in_place=True)
                    except jsonpatch.JsonPatchConflict as e:
                        LOGGER.info(
                            "Patch already applied %s: %s",
                            os.path.join(append_dir, file_path),
                            str(e),
                        )
                    except jsonpatch.JsonPatchTestFailed as e:
                        LOGGER.info(
                            "Patch test failed %s: %s",
                            os.path.join(append_dir, file_path),
                            str(e),
                        )
                    except jsonpatch.JsonPatchException as e:
                        LOGGER.info(
                            "Patch exception raised for %s: %s",
                            os.path.join(append_dir, file_path),
                            str(e),
                        )
                    except jsonpointer.JsonPointerException as e:
                        LOGGER.info(
                            "Patch exception with pointer %s: %s",
                            os.path.join(append_dir, file_path),
                            str(e),
                        )
                    except Exception as e:  # pylint: disable=broad-exception-caught
                        LOGGER.info(
                            "Unknown exception raised applying patch %s: %s",
                            os.path.join(append_dir, file_path),
                            str(e),
                        )

        return content

    def patch(self, patch: SchemaPatch, region: str) -> None:
        """Patch the schemas as needed

        Args:
            patch: The patches to be applied to the schemas
        Returns:
            None: Returns when completed
        """

        resource_types = []
        all_resource_types = self.get_resource_types(region)[:]
        # Remove unsupported resource using includes
        if patch.included_resource_types:
            for include in patch.included_resource_types:
                regex = re.compile(include.replace("*", "(.*)") + "$")
                matches = [
                    string for string in all_resource_types if re.match(regex, string)
                ]

                resource_types.extend(matches)
        else:
            resource_types = all_resource_types[:]

        # Remove unsupported resources using the excludes
        for exclude in patch.excluded_resource_types:
            regex = re.compile(exclude.replace("*", "(.*)") + "$")
            matches = [string for string in resource_types if re.match(regex, string)]
            for match in matches:
                resource_types.remove(match)

        # Remove unsupported resources
        removed_types = list(set(all_resource_types) - set(resource_types))
        if removed_types:
            for removed_type in removed_types:
                self._removed_types.append(removed_type)
            self.get_resource_schema.cache_clear()
            self.get_resource_types.cache_clear()

        for resource_type, patches in patch.patches.items():
            try:
                schema = self.get_resource_schema(
                    resource_type=resource_type, region=region
                )
            except ResourceNotFoundError:
                # Resource type doesn't exist in this region
                continue
            try:
                schema.patch(patches=patches)
            except Exception as e:
                print(f"Error applying patch {patches} for {resource_type}: {e}")
                sys.exit(1)

    @lru_cache(maxsize=None)
    def get_type_getatts(self, resource_type: str, region: str) -> AttributeDict:
        """Get the GetAtts for a type in a region

        Args:
            resource_type: The type of the resource. Example: AWS::S3::Bucket
            region: The region to load the resource type from
        Returns:
            Dict(str, Dict): Returns a Dict where the keys are the attributes and the
                value is the CloudFormation schema description of the attribute
        """
        resource_type = self._normalize_resource_type(resource_type)
        self.get_resource_schema(region=region, resource_type=resource_type)
        return self._schemas[region][resource_type].get_atts

    @lru_cache(maxsize=None)
    def get_type_ref(self, resource_type: str, region: str) -> dict[str, Any]:
        """Get the Ref information for a type in a region

        Args:
            resource_type: The type of the resource. Example: AWS::S3::Bucket
            region: The region to load the resource type from
        Returns:
            dict(str, Any): Returns a Dict where the keys are the attributes and the
                value is the CloudFormation schema description of the attribute
        """
        resource_type = self._normalize_resource_type(resource_type)
        self.get_resource_schema(region=region, resource_type=resource_type)
        return self._schemas[region][resource_type].ref


PROVIDER_SCHEMA_MANAGER: ProviderSchemaManager = ProviderSchemaManager()



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from test.integration import BaseCliTestCase

from cfnlint import ConfigMixIn


class TestQuickStartTemplates(BaseCliTestCase):
    """Test QuickStart Templates Parsing"""

    scenarios = [
        {
            "filename": (
                "test/fixtures/templates/integration/resources-cloudformation-init.yaml"
            ),
            "results_filename": (
                "test/fixtures/results/integration/resources-cloudformation-init.json"
            ),
            "exit_code": 0,
        },
        {
            "filename": "test/fixtures/templates/integration/dynamic-references.yaml",
            "results_filename": (
                "test/fixtures/results/integration/dynamic-references.json"
            ),
            "exit_code": 2,
        },
        {
            "filename": "test/fixtures/templates/integration/ref-no-value.yaml",
            "results_filename": "test/fixtures/results/integration/ref-no-value.json",
            "exit_code": 2,
        },
        {
            "filename": "test/fixtures/templates/integration/metdata.yaml",
            "results_filename": "test/fixtures/results/integration/metadata.json",
            "exit_code": 4,
        },
        {
            "filename": "test/fixtures/templates/integration/availability-zones.yaml",
            "results_filename": (
                "test/fixtures/results/integration/availability-zones.json"
            ),
            "exit_code": 2,
        },
        {
            "filename": "test/fixtures/templates/integration/getatt-types.yaml",
            "results_filename": "test/fixtures/results/integration/getatt-types.json",
            "exit_code": 10,
        },
        {
            "filename": "test/fixtures/templates/integration/ref-types.yaml",
            "results_filename": "test/fixtures/results/integration/ref-types.json",
            "exit_code": 2,
        },
        {
            "filename": "test/fixtures/templates/integration/formats.yaml",
            "results_filename": "test/fixtures/results/integration/formats.json",
            "exit_code": 2,
        },
        {
            "filename": (
                "test/fixtures/templates/integration/aws-ec2-networkinterface.yaml"
            ),
            "results_filename": (
                "test/fixtures/results/integration/aws-ec2-networkinterface.json"
            ),
            "exit_code": 2,
        },
        {
            "filename": "test/fixtures/templates/integration/aws-ec2-instance.yaml",
            "results_filename": (
                "test/fixtures/results/integration/aws-ec2-instance.json"
            ),
            "exit_code": 2,
        },
        {
            "filename": (
                "test/fixtures/templates/integration/aws-ec2-launchtemplate.yaml"
            ),
            "results_filename": (
                "test/fixtures/results/integration/aws-ec2-launchtemplate.json"
            ),
            "exit_code": 2,
        },
        {
            "filename": "test/fixtures/templates/integration/aws-ec2-subnet.yaml",
            "results_filename": "test/fixtures/results/integration/aws-ec2-subnet.json",
            "exit_code": 2,
        },
        {
            "filename": "test/fixtures/templates/integration/aws-dynamodb-table.yaml",
            "results_filename": (
                "test/fixtures/results/integration/aws-dynamodb-table.json"
            ),
            "exit_code": 2,
        },
    ]

    def test_templates(self):
        """Test same templates using integration approach"""
        self.run_module_integration_scenarios(
            ConfigMixIn(
                [],
                include_checks=["I"],
                include_experimental=True,
            )
        )



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from __future__ import annotations

from collections import deque
from test.unit.rules import BaseRuleTestCase

from cfnlint.jsonschema import ValidationError
from cfnlint.rules.jsonschema.Base import BaseJsonSchema


class TestBaseJsonSchemaUnsetHandling(BaseRuleTestCase):
    """Test Base Json Schema unset value handling"""

    def test_convert_validation_errors_with_unset_values(self):
        """Test that unset validator and instance values are not added to kwargs"""

        class Rule(BaseJsonSchema):
            id = "E3UNSET"

        rule = Rule()

        # Test case 1: Both validator and instance are unset (using defaults)
        error_both_unset = ValidationError(
            "Test error with both unset",
            path=deque(["test"]),
            # Don't pass validator or instance - they default to _unset
        )

        matches = rule._convert_validation_errors_to_matches(["root"], error_both_unset)

        self.assertEqual(len(matches), 1)
        match = matches[0]

        # Check that validator and instance are not in the match attributes
        self.assertFalse(hasattr(match, "validator"))
        self.assertFalse(hasattr(match, "instance"))

    def test_convert_validation_errors_with_set_values(self):
        """Test that set validator and instance values are added to kwargs"""

        class Rule(BaseJsonSchema):
            id = "E3UNSET"

        rule = Rule()

        # Test case: Both validator and instance are set
        error_both_set = ValidationError(
            "Test error with both set",
            path=deque(["test"]),
            validator="type",
            validator_value="string",
            instance=123,
        )

        matches = rule._convert_validation_errors_to_matches(["root"], error_both_set)

        self.assertEqual(len(matches), 1)
        match = matches[0]

        # Check that validator is in the match attributes
        # (instance is intentionally excluded)
        self.assertTrue(hasattr(match, "validator"))
        self.assertEqual(match.validator, "type")
        # Instance is not included to avoid large data in matches
        self.assertFalse(hasattr(match, "instance"))

    def test_convert_validation_errors_mixed_set_unset(self):
        """Test mixed scenarios where one value is set and another is unset"""

        class Rule(BaseJsonSchema):
            id = "E3UNSET"

        rule = Rule()

        # Test case 1: validator is set, instance is unset (default)
        error_validator_set = ValidationError(
            "Test error with validator set",
            path=deque(["test1"]),
            validator="enum",
            # instance defaults to _unset
        )

        matches = rule._convert_validation_errors_to_matches(
            ["root"], error_validator_set
        )

        self.assertEqual(len(matches), 1)
        match = matches[0]

        self.assertTrue(hasattr(match, "validator"))
        self.assertFalse(hasattr(match, "instance"))
        self.assertEqual(match.validator, "enum")

    def test_convert_validation_errors_with_extra_args_and_unset(self):
        """Test that extra_args work correctly when validator/instance are unset"""

        class Rule(BaseJsonSchema):
            id = "E3UNSET"

        rule = Rule()

        error = ValidationError(
            "Test error with extra args and unset values",
            path=deque(["test"]),
            # validator and instance default to _unset
            extra_args={"custom_field": "custom_value", "another_field": 42},
        )

        matches = rule._convert_validation_errors_to_matches(["root"], error)

        self.assertEqual(len(matches), 1)
        match = matches[0]

        # Check that validator and instance are not present
        self.assertFalse(hasattr(match, "validator"))
        self.assertFalse(hasattr(match, "instance"))

        # Check that extra_args are still present
        self.assertTrue(hasattr(match, "custom_field"))
        self.assertTrue(hasattr(match, "another_field"))
        self.assertEqual(match.custom_field, "custom_value")
        self.assertEqual(match.another_field, 42)

    def test_convert_validation_errors_with_context_and_unset(self):
        """Test that context errors also handle unset values correctly"""

        class Rule(BaseJsonSchema):
            id = "E3UNSET"

        rule = Rule()

        # Create context errors with mixed set/unset values
        context_error1 = ValidationError(
            "Context error 1 - validator unset",
            path=deque(["context1"]),
        )

        context_error2 = ValidationError(
            "Context error 2 - instance unset",
            path=deque(["context2"]),
            validator="type",
            # instance defaults to _unset
        )

        # Main error with context
        main_error = ValidationError(
            "Main error",
            path=deque(["main"]),
            validator="anyOf",
            context=[context_error1, context_error2],
        )

        matches = rule._convert_validation_errors_to_matches(["root"], main_error)

        self.assertEqual(len(matches), 1)
        main_match = matches[0]

        # Check main match has both validator and instance
        self.assertTrue(hasattr(main_match, "validator"))
        self.assertEqual(main_match.validator, "anyOf")

        # Check context matches
        self.assertEqual(len(main_match.context), 2)

        # Context match 1: validator unset, instance set
        context_match1 = main_match.context[0]
        self.assertFalse(hasattr(context_match1, "validator"))

        # Context match 2: validator set, instance unset
        context_match2 = main_match.context[1]
        self.assertTrue(hasattr(context_match2, "validator"))
        self.assertEqual(context_match2.validator, "type")



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

import logging
from io import StringIO
from test.testlib.testcase import BaseTestCase
from unittest.mock import patch

from cfnlint import ConfigMixIn
from cfnlint.helpers import format_json_string
from cfnlint.runner import Runner

LOGGER = logging.getLogger("cfnlint")


class TestCli(BaseTestCase):
    """Test CLI with config"""

    def tearDown(self):
        """Setup"""
        for handler in LOGGER.handlers:
            LOGGER.removeHandler(handler)

    @patch("cfnlint.maintenance.update_documentation")
    def test_update_documentation(self, mock_maintenance):
        config = ConfigMixIn(["--update-documentation"])

        runner = Runner(config)

        with self.assertRaises(SystemExit) as e:
            runner.cli()

        self.assertEqual(e.exception.code, 0)
        mock_maintenance.assert_called_once()

    @patch("cfnlint.maintenance.update_resource_specs")
    def test_update_specs(self, mock_maintenance):
        mock_maintenance.return_value = 0
        config = ConfigMixIn(["--update-specs"])

        runner = Runner(config)

        with self.assertRaises(SystemExit) as e:
            runner.cli()

        self.assertEqual(e.exception.code, 0)
        mock_maintenance.assert_called_once()

    @patch("cfnlint.maintenance.patch_resource_specs")
    def test_patch_specs(self, mock_maintenance):
        config = ConfigMixIn(["--patch-specs"])

        runner = Runner(config)

        with self.assertRaises(SystemExit) as e:
            runner.cli()

        self.assertEqual(e.exception.code, 0)
        mock_maintenance.assert_called_once()

    @patch("cfnlint.maintenance.update_iam_policies")
    def test_update_iam_policies(self, mock_maintenance):
        config = ConfigMixIn(["--update-iam-policies"])

        runner = Runner(config)

        with self.assertRaises(SystemExit) as e:
            runner.cli()

        self.assertEqual(e.exception.code, 0)
        mock_maintenance.assert_called_once()

    def test_list_rules(self):
        config = ConfigMixIn(["--list-rules"])

        runner = Runner(config)

        with self.assertRaises(SystemExit) as e:
            runner.cli()

        self.assertEqual(e.exception.code, 0)

    @patch("argparse.ArgumentParser.print_help")
    @patch("sys.stdin.isatty")
    def test_print_help(self, mock_isatty, mock_print_help):
        config = ConfigMixIn([])

        runner = Runner(config)
        mock_isatty.return_value = True
        with self.assertRaises(SystemExit) as e:
            runner.cli()

        self.assertEqual(e.exception.code, 1)
        mock_print_help.assert_called_once()
        # isatty should be called at least once
        self.assertTrue(mock_isatty.call_count >= 1)
        # self.assertEqual(mock_isatty.call_count, 2)

    @patch("argparse.ArgumentParser.print_help")
    @patch("sys.stdin.isatty")
    def test_no_templates_no_deployment_files(self, mock_isatty, mock_print_help):
        """Test that help is printed when no templates or deployment
        files are provided and stdin is a tty"""
        # Create a config with no templates or deployment files
        config = ConfigMixIn([])

        # Ensure templates and deployment_files are empty or None
        self.assertTrue(not config.templates or config.templates == [])
        self.assertTrue(not config.deployment_files or config.deployment_files == [])

        runner = Runner(config)
        mock_isatty.return_value = True

        # Should exit with code 1 and print help
        with self.assertRaises(SystemExit) as e:
            runner.cli()

        self.assertEqual(e.exception.code, 1)
        mock_print_help.assert_called_once()
        # isatty should be called at least once
        self.assertTrue(mock_isatty.call_count >= 1)

    @patch("argparse.ArgumentParser.print_help")
    @patch("sys.stdin.isatty")
    @patch("cfnlint.runner.Runner._cli_output")
    def test_no_templates_no_deployment_files_with_stdin(
        self, mock_cli_output, mock_isatty, mock_print_help
    ):
        """Test that when no templates or deployment files are
        provided but stdin is not a tty, the program continues
        execution without printing help"""
        # Create a config with no templates or deployment files
        config = ConfigMixIn([])

        # Ensure templates and deployment_files are empty or None
        self.assertTrue(not config.templates or config.templates == [])
        self.assertTrue(not config.deployment_files or config.deployment_files == [])

        # Mock _cli_output to avoid actual processing
        mock_cli_output.return_value = None

        runner = Runner(config)
        mock_isatty.return_value = False

        # Should not exit and not print help
        runner.cli()

        # Help should not be printed
        mock_print_help.assert_not_called()
        # isatty should be called at least once
        self.assertTrue(mock_isatty.call_count >= 1)
        # _cli_output should be called once
        mock_cli_output.assert_called_once()

    def test_bad_regions(self):
        config = ConfigMixIn(
            [
                "--regions",
                "us-north-5",
                "--template",
                "test/fixtures/templates/good/generic.yaml",
            ]
        )

        runner = Runner(config)

        with self.assertRaises(SystemExit) as e:
            runner.cli()

        self.assertEqual(e.exception.code, 32)

    @patch("argparse.ArgumentParser.print_help")
    def test_templates_with_deployment_files(self, mock_print_help):
        config = ConfigMixIn(
            [
                "--template",
                "test/fixtures/templates/good/generic.yaml",
            ],
            deployment_files=[
                "test/fixtures/templates/good/generic.yaml",
            ],
        )

        runner = Runner(config)

        with self.assertRaises(SystemExit) as e:
            runner.cli()

        self.assertEqual(e.exception.code, 1)
        mock_print_help.assert_called_once()

    @patch("argparse.ArgumentParser.print_help")
    def test_templates_with_parameters_and_parameter_files(self, mock_print_help):
        config = ConfigMixIn(
            [
                "--template",
                "test/fixtures/templates/good/generic.yaml",
            ],
            parameters=[
                {"foo": "bar"},
            ],
            parameter_files=["test/fixtures/parameter_files/*.json"],
        )

        runner = Runner(config)

        with self.assertRaises(SystemExit) as e:
            runner.cli()

        self.assertEqual(e.exception.code, 1)
        mock_print_help.assert_called_once()

    @patch("argparse.ArgumentParser.print_help")
    def test_templates_with_deployment_files_and_parameters(self, mock_print_help):
        config = ConfigMixIn(
            [],
            parameters=[
                {"foo": "bar"},
            ],
            deployment_files=["test/fixtures/parameter_files/*.json"],
        )

        runner = Runner(config)

        with self.assertRaises(SystemExit) as e:
            runner.cli()

        self.assertEqual(e.exception.code, 1)
        mock_print_help.assert_called_once()

    @patch("argparse.ArgumentParser.print_help")
    def test_templates_with_parameters_and_multiple_templates(self, mock_print_help):
        config = ConfigMixIn(
            [
                "--template",
                "test/fixtures/templates/good/generic.yaml",
                "test/fixtures/templates/good/generic.yaml",
            ],
            parameters=[
                {"foo": "bar"},
            ],
        )

        runner = Runner(config)

        with self.assertRaises(SystemExit) as e:
            runner.cli()

        self.assertEqual(e.exception.code, 1)
        mock_print_help.assert_called_once()

    @patch("fileinput.input")
    @patch("sys.stdin.isatty")
    def test_templates_with_stdin(self, mock_isatty, mock_fileinput):
        template = {
            "Resources": {
                "Bucket": {
                    "Type": "AWS::S3::Bucket",
                }
            }
        }

        mock_fileinput.return_value = StringIO(format_json_string(template))
        mock_isatty.return_value = False

        config = ConfigMixIn()

        runner = Runner(config)

        with self.assertRaises(SystemExit) as e:
            runner.cli()

        self.assertEqual(e.exception.code, 0)

    @patch("fileinput.input")
    @patch("sys.stdin.isatty")
    def test_templates_with_stdin_with_bad_syntax(self, mock_isatty, mock_fileinput):
        template = "{"

        mock_fileinput.return_value = StringIO(template)
        mock_isatty.return_value = False

        config = ConfigMixIn()

        runner = Runner(config)

        with self.assertRaises(SystemExit) as e:
            runner.cli()

        self.assertEqual(e.exception.code, 2)



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from __future__ import annotations

import fnmatch
import json
import os
import pathlib
from typing import Any, List
from unittest import TestCase

import pytest
import regex as re

import cfnlint
from cfnlint.helpers import ensure_list, load_plugins, load_resource
from cfnlint.jsonschema import StandardValidator, ValidationError
from cfnlint.schema.resolver import RefResolutionError, RefResolver


@pytest.mark.data
class TestSchemaFiles(TestCase):
    """Test schema files"""

    _found_keywords: List[str] = [
        "*",
        "Conditions",
        "Description",
        "Mappings",
        "Metadata",
        "Metadata/AWS::CloudFormation::Interface",
        "Metadata/cfn-lint",
        "Outputs",
        "Outputs/*",
        "Outputs/*/Condition",
        "Outputs/*/Export/Name",
        "Outputs/*/Value",
        "Parameters",
        "Parameters/*",
        "Parameters/*/AllowedPattern",
        "Resources",
        "Resources/*",
        "Resources/*/Condition",
        "Resources/*/CreationPolicy",
        "Resources/*/DeletionPolicy",
        "Resources/*/DependsOn",
        "Resources/*/DependsOn/*",
        "Resources/*/Metadata",
        "Resources/*/Metadata/AWS::CloudFormation::Init",
        "Resources/*/Type",
        "Resources/*/UpdatePolicy",
        "Resources/*/UpdateReplacePolicy",
        "Rules",
        "Rules/*/Assertions/*/Assert",
        "Rules/*/RuleCondition",
        "Transform",
    ]

    def setUp(self) -> None:
        schema_path = os.path.join(os.path.dirname(cfnlint.__file__), "data", "schemas")
        self.paths = {
            "extensions": os.path.join(schema_path, "extensions"),
            "providers": os.path.join(schema_path, "providers"),
            "resources": os.path.join(schema_path, "resources"),
            "other": os.path.join(schema_path, "other"),
            "fixtures": os.path.join(
                os.path.dirname(__file__),
                "..",
                "fixtures",
                "schemas",
                "providers",
            ),
        }
        filename = os.path.join(
            self.paths["fixtures"], "..", "json_schema", "draft7.json"
        )
        with open(filename, "r") as fh:
            d = json.load(fh)
            self.schema_draft7 = d

        super().setUp()

    def get_files(self, dir):
        for dirpath, _, filenames in os.walk(dir):
            for filename in fnmatch.filter(filenames, "*.json"):
                yield dirpath, filename

    def pattern(self, validator, patrn, instance, schema):
        try:
            re.compile(patrn)
        except Exception:
            yield ValidationError(f"Pattern doesn't compile: {patrn}")

    def validate_basic_schema_details(
        self, schema_resolver: RefResolver, filepath: str
    ):
        """
        Validate that readOnly, writeOnly, etc are valid
        """
        sections = [
            "readOnlyProperties",
            "writeOnlyProperties",
            "conditionalCreateOnlyProperties",
            "nonPublicProperties",
            "nonPublicDefinitions",
            "createOnlyProperties",
            "deprecatedProperties",
            "primaryIdentifier",
        ]
        for section in sections:
            for prop in schema_resolver.referrer.get(section, []):
                try:
                    self.assertIsNotNone(schema_resolver.resolve_cfn_pointer(prop))
                except RefResolutionError:
                    self.fail(f"Can't find prop {prop} for {section} in {filepath}")

    def _build_keywords(self, obj: Any, schema_resolver: RefResolver, refs: list[str]):
        if not isinstance(obj, dict):
            yield []
            return

        if "$ref" in obj:
            ref = obj["$ref"]
            if ref in refs:
                yield []
                return
            _, resolved_schema = schema_resolver.resolve(ref)
            yield from self._build_keywords(
                resolved_schema, schema_resolver, refs + [ref]
            )

        if "type" in obj:
            if "object" in ensure_list(obj["type"]):
                if "properties" in obj:
                    for k, v in obj["properties"].items():
                        for item in self._build_keywords(v, schema_resolver, refs):
                            yield [k] + item
            if "array" in ensure_list(obj["type"]):
                if "items" in obj:
                    for item in self._build_keywords(
                        obj["items"], schema_resolver, refs
                    ):
                        yield ["*"] + item

        yield []

    def build_keywords(self, schema_resolver):
        self._found_keywords.append(
            "/".join(["Resources", schema_resolver.referrer["typeName"], "Properties"])
        )
        for k, v in schema_resolver.referrer.get("properties").items():
            for item in self._build_keywords(v, schema_resolver, []):
                self._found_keywords.append(
                    "/".join(
                        [
                            "Resources",
                            schema_resolver.referrer["typeName"],
                            "Properties",
                            k,
                        ]
                        + item
                    )
                )

    def test_data_module_specs(self):
        """Test data file formats"""

        draft7_schema = load_resource(
            "cfnlint.data.schemas.other.draft7", "schema.json"
        )
        store = {"http://json-schema.org/draft-07/schema": draft7_schema}
        dir = self.paths["fixtures"]
        for dirpath, filename in self.get_files(dir):
            with open(os.path.join(dirpath, filename), "r", encoding="utf8") as fh:
                store[filename] = json.load(fh)

        resolver = RefResolver.from_schema(
            store["provider.definition.schema.v1.json"], store=store
        )

        validator = (
            StandardValidator({})
            .extend(
                validators={
                    "cfnLint": self.cfn_lint,
                    "pattern": self.pattern,
                },
            )(schema=store["provider.definition.schema.v1.json"])
            .evolve(resolver=resolver)
        )

        # Only process us-east-1 region for keyword building (like original test)
        region = "us-east-1"
        provider_module = region.replace("-", "_")
        try:
            provider = __import__(
                f"cfnlint.data.schemas.providers.{provider_module}", fromlist=["types"]
            )
            resource_types = getattr(provider, "types", {})
        except (ImportError, AttributeError):
            self.fail(f"Could not load provider module for {region}")

        # Load each resource schema
        for resource_type, schema_hash in resource_types.items():
            schema_path = os.path.join(self.paths["resources"], f"{schema_hash}.json")

            if not os.path.exists(schema_path):
                continue

            with open(schema_path, "r", encoding="utf8") as fh:
                d = json.load(fh)
                # not allowed but true with this resource
                if resource_type == "AWS::CloudFormation::CustomResource":
                    d["additionalProperties"] = False
                # Skip synthetic types that don't follow standard schema
                if resource_type in ("Module", "AWS::CDK::Metadata"):
                    continue
                errs = list(validator.iter_errors(d))
                self.assertListEqual(
                    errs, [], f"Error with {resource_type} ({schema_hash}): {errs}"
                )
                schema_resolver = RefResolver(d)
                self.validate_basic_schema_details(
                    schema_resolver, f"{resource_type} ({schema_hash})"
                )

                # Build keywords for us-east-1 schemas
                self.build_keywords(schema_resolver)

    def cfn_lint(self, validator, _, keywords, schema):
        keywords = ensure_list(keywords)
        self._found_keywords.extend(keywords)

    def test_other_specs(self):
        """Test data file formats"""

        store = {}
        dir = self.paths["fixtures"]
        for dirpath, filename in self.get_files(dir):
            with open(os.path.join(dirpath, filename), "r", encoding="utf8") as fh:
                store[filename] = json.load(fh)

        resolver = RefResolver.from_schema(store["cfnlint.schema.v1.json"], store=store)

        validator = (
            StandardValidator({})
            .extend(
                validators={
                    "cfnLint": self.cfn_lint,
                    "pattern": self.pattern,
                },
            )(schema=store["cfnlint.schema.v1.json"])
            .evolve(resolver=resolver)
        )

        for dir_name in ["extensions", "other"]:
            dir = self.paths[dir_name]

            for dirpath, filename in self.get_files(dir):
                with open(os.path.join(dirpath, filename), "r", encoding="utf8") as fh:
                    d = json.load(fh)
                    errs = list(validator.iter_errors(d))
                    self.assertListEqual(
                        errs, [], f"Error with {dirpath}/{filename}: {errs}"
                    )
                    schema_resolver = RefResolver(d)
                    self.validate_basic_schema_details(
                        schema_resolver, f"{dirpath}/{filename}"
                    )

    def test_all_referenced_hashes_exist(self):
        """Test that all hashes referenced by provider files
        have corresponding schema files
        """
        import importlib
        from pathlib import Path

        # Collect all hashes referenced by provider files
        referenced_hashes = set()
        providers_dir = Path(self.paths["providers"])

        for provider_file in providers_dir.glob("*.py"):
            if provider_file.name == "__init__.py":
                continue

            region_name = provider_file.stem
            try:
                module_name = f"cfnlint.data.schemas.providers.{region_name}"
                provider = importlib.import_module(module_name)
                if hasattr(provider, "types"):
                    for resource_type, schema_hash in provider.types.items():
                        referenced_hashes.add((schema_hash, resource_type, region_name))
            except (ImportError, AttributeError):
                continue

        # Check all referenced hashes have corresponding files
        resources_dir = Path(self.paths["resources"])
        missing_files = []

        for schema_hash, resource_type, region_name in referenced_hashes:
            schema_file = resources_dir / f"{schema_hash}.json"
            if not schema_file.exists():
                missing_files.append(
                    f"{resource_type} ({schema_hash}) in {region_name}"
                )

        self.assertEqual(
            missing_files,
            [],
            f"Found {len(missing_files)} missing schema files: {missing_files[:10]}...",
        )

    def test_all_resource_files_referenced(self):
        """Test that all resource schema files are
        referenced by at least one provider
        """
        import importlib
        from pathlib import Path

        # Collect all hashes referenced by provider files
        referenced_hashes = set()
        providers_dir = Path(self.paths["providers"])

        for provider_file in providers_dir.glob("*.py"):
            if provider_file.name == "__init__.py":
                continue

            region_name = provider_file.stem
            try:
                module_name = f"cfnlint.data.schemas.providers.{region_name}"
                provider = importlib.import_module(module_name)
                if hasattr(provider, "types"):
                    referenced_hashes.update(provider.types.values())
            except (ImportError, AttributeError):
                continue

        # Check all resource files are referenced
        resources_dir = Path(self.paths["resources"])
        orphaned_files = []

        for schema_file in resources_dir.glob("*.json"):
            schema_hash = schema_file.stem
            if schema_hash not in referenced_hashes:
                orphaned_files.append(schema_hash)

        self.assertEqual(
            orphaned_files,
            [],
            (
                f"Found {len(orphaned_files)} orphaned "
                f"resource files: {orphaned_files[:10]}..."
            ),
        )

    def test_x_keywords(self):
        root_dir = pathlib.Path(__file__).parent.parent.parent / "src/cfnlint/rules"
        rules = load_plugins(
            str(root_dir),
            "CfnLintKeyword",
            "cfnlint.rules.jsonschema.CfnLintKeyword",
        )
        rules.extend(
            load_plugins(
                str(root_dir),
                "CfnLintJsonSchema",
                "cfnlint.rules.jsonschema.CfnLintJsonSchema",
            )
        )

        for rule in rules:
            for keyword in rule.keywords:
                self.assertIn(keyword, self._found_keywords, f"{keyword} not found")
